#--> +++++++++++++++++++++++++++++++++++++++++++++++++++++++ <--#
#--> check_resource_settings.tcl                             <--#
#-->                                                         <--#
#--> Routine to check to see that you have a spectrum server <--#
#--> running and that you have the correct resources         <--#
#--> claimed, if these are not then it will launch/set them  <--#
#--> for you .........                                       <--#
#-->                                                         <--#
#--> +++++++++++++++++++++++++++++++++++++++++++++++++++++++ <--#
proc check_resource_settings {} {
    global mydebug
    global env

    #--> Just try to launch the spectrum server ... if it exists 
    #--> then this will fail and the user will be none the 
    #--> wiser (:>>
    set progname "/MIDAS/bin_SunOS/memsas"
    #--> catch the launch ....
#    set status [ catch { exec $progname "-p-1" } ]
    #--> Ok now lets do the difficult stuff .. check the resources
    set resources {}
    set resources [ :experiment resources ]
    #--> Now look for our killer app ...  localsas
    set present    0
    set name       ""
    set capability ""
    foreach res $resources {
	foreach r $res {
	    switch [ first $r ] {
		name          { set name [ second $r ] }
		capability    { set cap  [ second $r ] }
	    }
	    if { $mydebug == 1 } then {
		puts $r
	    }
	    if { $name == "localsas" } then {
		set present 1
		if { $capability != "0x0000000000000000" } then {
		    set present 2
		}
	    }
	}
    }
    #--> Ok is the resource that we need present ?
    if { $present == 1 } then {
	#--> Present but bad capeability
	if { $mydebug == 1 } then {
	    puts "localsas is present but the capability is not correct"
	}
    } elseif { $present == 2 } then {
	#--> Present and all is well
	if { $mydebug == 1 } then {
	    puts "localsas is present and we do not need to do anything"
	}
    } else {
	#--> Not present so we need to install ..
	if { $mydebug == 1 } then {
	    puts "localsas not installed"
	}
	set directory [file join $env(MIDASTempDir) tcl[pid]]
	set file "localsas"
	if { $mydebug == 1 } then {
	    puts "directory is : $directory"
	    puts "file is      : $file"
	}
	#--> Create apparatus file
	set appfile [file join $directory $file]
	set z [catch {open $appfile w} f]
	if {$z != 0} then {
	    midas-warning "Unable to write [file join $directory $file]\n$f"
	    return
	}
	puts $f "((name $file)"
	puts $f " (host localhost)"
	puts $f " (role localsas)"
	puts $f " (type spectrum)"
	puts $f " (ipaddress 123456)"
	puts $f " (port 10230))"
	close $f
	if { $mydebug == 1 } then {
	    puts "Appfile      : $appfile "
	}
	#--> Attach the begger
	:apparatus attach :path $directory :name $file
	#--> Suppose we had better claim things
	:apparatus claim :name $file
    }

}