# No #! line here, since this is actually started from another
# script file that processes debugging options and the like.
# That file also contains all the settings relating to the particular
# installation.
set mainGeometry +10+10
set userDefaultDir [file join ~ .newed]

# Get the progress meter up.  I keep all the private variables
# relating to the meter in a namespace, so as to avoid cluttering the
# global namespace up with variables that aren't that interesting,
# even when debugging.
namespace eval LoadingProgressMeter {
    # How many steps the progress moves in, as a floating-point
    # number (so the divisions happen right.)  Set by hand to the
    # value reported if the system doesn't work right.
    set max 58.0

    wm title            . "Loading Example"
    wm protocol         . WM_DELETE_WINDOW {#Do nothing}
    wm overrideredirect . 1
    wm positionfrom     . user
    wm geometry         . +100+100
    tk appname "Example"

    set gbg #E0FFE0;           # Greenish background colour
    set big   {Helvetica -24 {bold italic}}
    set mid   {Helvetica -18}
    set small {Helvetica -10}; # Not actually used here...
    set load  {Times -12}

    # All parameters are set by hand, since I've not loaded any
    # app. defaults files at this point.
    . conf  -bg $gbg  -cursor watch
    label .title -text "    Example Loading   "  -font $big \
            -bg $gbg  -fg red3  -padx 0
    label .loadmsg  -textvariable loadtext  -bd 2  -relief ridge \
            -anchor w  -bg $gbg  -font $load
    frame .progress  -relief ridge  -bd 2  -bg $gbg

    # The progress bar is a neutered scale widget.  It would also be
    # possible to use a canvas - I didn't though, as scales have less
    # in the way of hardwired (in the Tk library) event-handling.
    scale .progress.bar  -from 0  -to 1  -label {}  -bd 0 \
            -orient horizontal  -length 20  -showvalue 0 \
            -background #000040  -troughcolor $gbg  -state normal \
            -tickinterval 0  -width 10  -takefocus 0  -cursor {} \
            -relief flat  -sliderrelief flat  -sliderlength 4
    .progress.bar set 0
    # The neutering is done by removing all the useful bindings from
    # the widget using a quick bindtags invokation.
    bindtags .progress.bar {. all}
    pack .progress.bar -fill x -expand 1

    grid .title   -          -padx 1.5m -pady 1m -sticky ew
    if [winfo exist .evalmsg] {grid .evalmsg  -columnspan 3}
    grid .loadmsg .progress  -sticky nsew

    tkwait visibility .
    update idletasks

    #----------------------------------------------------------------
    variable count -1
    variable len 0
    proc updateprogress {args} {
        global loadtext
        variable count
        variable max

        incr count
        set width [winfo width .progress.bar]
        .progress.bar conf -sliderlength \
                [expr {int(($width-4)*$count/$max)+4}]
        if {$count%5 == 0} {
            update; # Update is slower - so only do it occasionally
        } else {
            update idletasks
        }
    }
    trace variable ::loadtext w [namespace code updateprogress]
    set ::loadtext ""
    #----------------------------------------------------------------
    namespace export check_progress_length
    proc check_progress_length {} {
        variable count
        variable max
        variable len
        if {$count!=$max} {
            puts stderr "[namespace current]::max not correctly\
                    adjusted - FIX to be [expr {$count-$len}]"
        }
        catch {unset loadtext}
    }
}
namespace import LoadingProgressMeter::*

if [info exist env(Load_List)] {
    foreach {sofile description} $env(Load_List) {
        set loadtext "Loading Object Code: $description"
        if [catch {load $sofile} s] {
            puts stderr "Failed to load $sofile\nPerhaps\
                    it should be built?\n$s"
            exit 1
        }
    }
}

# Load the application defaults files - the system one first...

# env(Lib_Dir) is the application library directory which is set in
# the calling shell script.
set loadtext "Loading resources"
option readfile [file join $env(Lib_Dir)  Example.ad] startupFile

# Now load the user app-defaults file
set loadtext "Loading user resources"
catch {
    # The catch is because this file might not exist...
    option readfile [file join $userDefaultDir NewEd.ad] userDefault
}

# The list of files to load.  This can be appended to during the
# loading process so that files can be split without needing to list
# all the parts here...
# 
# Note that comments are "filenames" that start with a #, and
# user-defined filenames start with a / or a ~
append loadlist {
    {# A few example files}
        foo.tcl bar.tcl spong/wibble.tcl
}

# Users can define additional code here.  This place is where users
# would add filenames to the loadlist.
set loadtext "Loading user setup: prior.tcl"
catch {source [file join $userDefaultDir prior.tcl]}

# Now we load the files in the loadlist 
for {
    set loadcount 0
} {
    $loadcount < [llength $loadlist]
} {
    incr loadcount
} {
    set f [lindex $loadlist $loadcount]
    switch [string index $f 0] {
        # {
            # Skip comments
            continue
        }
        ~ - / {
            # User-defined (absolute) filenames
            if [catch {source $f}] {
                puts stderr "While loading user file\
                        \"$f\"...\n$errorInfo"
                exit 1
            }
        }
        default {
            # Application-defined (relative) filenames
            if [catch {source [file join $env(Tcl_Dir) $f]}] {
                puts stderr "While loading application file\
                        \"$f\"...\n$errorInfo"
                exit 1
            }
        }
    }
}

# Another place for users to define code.  At this point, it is
# guaranteed to run after the application has been loaded.
set loadtext "Loading user setup: next.tcl"
catch {source [file join $userDefaultDir next.tcl]}

# Hide the splash-screen now we've finished with it.
wm withdraw .

# One of the files loaded has created (and withdrawn) the main
# application interface using the window name .main  Show it now that
# we've finished loading everything.  Then wait for the window to be
# destroyed before exiting.  The explicit exit makes sure that any
# cleanup-callbacks linked to exit (by rename/replace) are called at
# the end of execution.
wm deiconify .main
tkwait window .main
exit
