#--> +++++++++++++++++++++++++++++++++++++++++++++++++++++++ <--#
#--> memsas_ctrl.tcl : tcl routine that sets up a window     <--#
#-->                   that displays the properties of all   <--#
#-->                   of the memsas daemons that are        <--#
#-->                   running on the current machine.       <--#
#-->                                                         <--#
#--> D.Appelbe Daresbury NPG 01/05/04                        <--#
#-->                                                         <--#
#--> +++++++++++++++++++++++++++++++++++++++++++++++++++++++ <--#
proc memsas_ctrl {} {
    #--> The first thing to do is to gather all the required 
    #--> information.
    memsas_ctrl.get_data
    #--> Now we shall generate the GUI used to display the
    #--> info
    memsas_ctrl.make_frame
    #--> Now configure our tags
    memsas_ctrl.configure_tags
    #--> Now paint our data to the appropriate places
    memsas_ctrl.paint_alloc
    memsas_ctrl.paint_daemons
}

#--> +++++++++++++++++++++++++++++++++++++++++++++++++++++++ <--#
#--> Subroutines called by memsas_ctrl                       <--#
#--> +++++++++++++++++++++++++++++++++++++++++++++++++++++++ <--#
proc memsas_ctrl.get_data {} {
    global slmemalloc
    global sldaemons

    set slmemalloc     [ list "Name           Port     Capability" ]
    set name "None Running"
    set port 0
    set cap  "0x00000000000000"

    #--> Read our current resources
    set resources {}
    set resources [ :experiment resources ]
    #--> Now extract the required data
    foreach res $resources {
	foreach r $res {
	    switch [ first $r ] {
		name        { set name [ second $r ] }
		port        { set port [ second $r ] }
		capability  { set cap  [ second $r ] }
	    }
	}
	if { $name == "localsas" } then {
	    set tmptxt [ format "%-15s%-9s%-17s" $name $port $cap ]
	    lappend slmemalloc $tmptxt
	}
    }

    #--> Now get data about memsas daemons that are currently running
    set sldaemons      [ list "Port      PID       Owner" ]  
    set filelist       [ :directory files :path "/tmp" ]
    set portlist       {}
    set pidlist        {}
    set present        0
    set actr           0
    set bctr           0
    foreach f $filelist {
	set fname [ first $f ]
	set present 0
	set present [ string match "memsas.port.*" $fname ]
	if { $present == 1 } then {
	    lappend portlist $fname
	    set actr [ expr $actr + 1 ]
	}
	set present 0
	set present [ string match "memsas.pid.*" $fname ]
	if { $present == 1 } then {
	    lappend pidlist $fname
	    set bctr [ expr $bctr + 1 ]
	}
    }
    #--> do we have parity between the two ?
    if { $actr == $bctr } then {
	foreach port $portlist {
	    set z  [ catch { open [ file join "/tmp" $port ] r } f ]
	    if { $z == 0 } then {
		gets $f aa
		close $f
		foreach pid $pidlist {
		    set bb [ third [ split $pid "." ] ]
		    if { $aa == $bb } then {
			set myport [ third [ split $port "." ] ]
			set owner  [ file attributes [ file join "/tmp" $port ] -owner ]
			lappend sldaemons [ format "%-10s%-10s%-10s" $myport $aa $owner ]
		    }
		}
	    } else {
		midas-warning "Could not open file : $port"
	    }
	}
    } elseif { $actr > $bctr } then {
    } else {
    }
    

}

proc memsas_ctrl.make_frame {} {
    global slmemalloc
    global sldaemons

    :frame FMEMSAS :label "MIDASsort: Memsas Information/Control Frame" \
	:resize F :transient T
    :panel PMEMSAS :layout V :padx 5 :pady 5
    :layout H
    :message MMEMSAS1 :label "Allocated Resources" :colour "-fg black" \
	:bold T
    :next-row T
    :galley GMEMSAS1 :cols 50 :rows 4 :deaf T :font variable \
	:see 0 :scroll N
    :next-row T
    :message MMEMSAS2 :label "The following Memsas Daemons are running" :colour "-fg black" \
	:bold T
    :next-row T
    :panel PMEMSAS1 :layout L :padx 5 :pady 5
    :layout H
    :list LMEMSAS1 :cols 35 :rows 7  \
	:see 0 :scroll V
    :panel PMEMSAS2 :layout L :padx 5 :pady 5
    :layout V
    :button BMEMSAS1 :label "Kill Daemon"
    :next-row T
    :next-row T
    :button BMEMSAS3 :label "Refresh"
    :next-row T
    :next-row T
    :button BMEMSAS2 :label "Cancel"
    :show T
}

proc memsas_ctrl.paint_alloc {} {
    global slmemalloc

    :modify GMEMSAS1 :empty T
    set lctr 0
    foreach line $slmemalloc {
	if { $lctr == 0 } then {
	    :modify GMEMSAS1 :usingtag tagdisp1 :strings "$line\n" :see 0
	} else {
	    :modify GMEMSAS1 :usingtag tagdisp2 :strings "$line\n" :see 0
	}
	set lctr [ expr $lctr + 1 ]
	if { $lctr >= 2 } then {
	    set lctr 0
	}
    }
}

proc memsas_ctrl.paint_daemons {} {
    global sldaemons

    :modify LMEMSAS1 :empty T :strings $sldaemons :see 0

}

proc memsas_ctrl.leave {} {
    destroy-frame FMEMSAS
}

proc memsas_ctrl.select args {
    global mydebug
    global mc_port mc_pid mc_listval

    if { $mydebug == 1 } then { puts "memsas_ctrl.select : args       = $args" }
    set mc_listval [ first $args ]
    if { $mydebug == 1 } then { puts "memsas_ctrl.select : list_value = $mc_listval" }
    set mc_port [ first [ second $args ] ]
    if { $mydebug == 1 } then { puts "memsas_ctrl.select : mc_port    = $mc_port" }
    set mc_pid [ second [ second $args ] ]
    if { $mydebug == 1 } then { puts "memsas_ctrl.select : mc_pid     = $mc_pid" }

}

proc memsas_ctrl.kill_daemon {} {
    global mydebug
    global mc_port mc_pid mc_listval

    if { $mydebug == 1 } then { puts "memsas_ctrl.kill_daemon: Entered" }
    if { $mc_listval > 0 } then {
	set z [ catch { exec "kill" $mc_pid } ]
	if { $z == 0 } then {
	    memsas_ctrl.get_data
	    memsas_ctrl.paint_daemons
	} else {
	    midas-error "Could not kill daemon"
	}
    } else {
	midas-error "Can not delete this line !"
    }

    if { $mydebug == 1 } then { puts "memsas_ctrl.kill_daemon: Exit" }

}

proc memsas_ctrl.refresh {} {
    memsas_ctrl.get_data
    memsas_ctrl.paint_alloc
    memsas_ctrl.paint_daemons
}

proc memsas_ctrl.configure_tags {} {
    :modify GMEMSAS1 :createtag "tagdisp1 -foreground red"
    :modify GMEMSAS1 :createtag "tagdisp2 -foreground blue"
}