#--> +++++++++++++++++++++++++++++++++++++++++++++++++++++++ <--#
#--> check_resource_settings.tcl                             <--#
#-->                                                         <--#
#--> Routine to check to see that you have a spectrum server <--#
#--> running and that you have the correct resources         <--#
#--> claimed, if these are not then it will launch/set them  <--#
#--> for you .........                                       <--#
#-->                                                         <--#
#--> +++++++++++++++++++++++++++++++++++++++++++++++++++++++ <--#
proc check_server {} {
    global mydebug
    global env
    global localsasport
    global serverprog

    set filelist     [ :directory files :path "/tmp" ]
    set pidlist      {}
    set pidlistrun   {}
    set portlist     {}
    set status       0
    set serverprog   "memsas"
    set serverport   -1
    set localsasport 0
    set actr         0
    set bctr         0
    set sasown       0
    set sasrun       0
    set sasownctr    0
    set sasownlist   {}

    #--> First we need to look for the presence of any memsas
    #--> files in /tmp
    foreach f $filelist {
	set present 0
	set fname [ first $f ]
	set present [ string match "memsas.pid.*" $fname ]
	if { $present == 1 } then {
	    lappend pidlist $fname
	    set actr [ expr $actr +1 ]
	    if { $mydebug == 1 } then {
		puts "check_server  : actr = $actr"
	    }
	}
	set present 0
	set present [ string match "memsas.port.*" $fname ]
	if { $present == 1 } then {
	    lappend portlist $fname
	    set bctr [ expr $bctr +1 ]
	    if { $mydebug == 1 } then {
		puts "check_server  : bctr = $bctr"
	    }
	}
    }

    #--> Now check for parity between what we believe is running 
    #--> and the true state of affairs.
    if { $env(platform) == "windows" } then {
    } else {
	foreach port $portlist {
	    set z [ catch { open [ file join "/tmp" $port ] r } f ]
	    if { $z == 0 } then {
		if { $mydebug == 1 } then {
		    puts "check_server  : file /tmp/$port opened succesfully"
		}
		gets $f aa
		close $f
		if { $mydebug == 1 } then {
		    puts "check_server  : pid according to $port is $aa"
		}
		#--> Check to see if this process is running
		if { $mydebug == 1 } then { puts "check_server  : about to exec kill -USR1 $aa" }
		set zz [ catch { exec "kill" "-USR1" $aa } m ]
		if { $mydebug == 1 } then { puts "check_server  : Completed call to exec kill -USR1 $aa" }
		if { $zz != 0 } then {
		    if { $mydebug == 1 } then {
			puts "check_server  : pid $aa does not exist ... removing lock files"
		    }
		    set sasown [ file owned [ file join "/tmp" $port ] ]
		    if { $sasown == 1 } then {
			#--> The process does not exist so remove the locks
			set zzz [ catch { exec rm [ file join "/tmp" $port ] m } ]
			if { $zzz != 0 } then {
			    midas-warning "Could not remove lockfile : /tmp/$port"
			}
			set zzz [ catch { exec rm [ file join "/tmp" "memsas.pid.$aa" ] m } ]
			if { $zzz != 0 } then {
			    midas-warning "Could not remove lockfile : /tmp/memsas.pid.$aa"
			}
		    } else {
			if { $mydebug == 1 } then {
			    puts "check_server  : Do not own file $port"
			}
			set sasrun 1
		    }
		} else {
		    #--> Ok so we are running but do we own a memsas?
		    if { $mydebug == 1 } then {
			puts "check_server  : Checking to see if we own any of the running memsas daemons"
		    }
		    set sasown 0
		    set sasown [ file owned [ file join "/tmp" $port ] ]
		    if { $sasown == 1 } then {
			set sasownctr [ expr $sasownctr +1 ]
			lappend sasownlist $port
			if { $mydebug == 1 } then {
			    puts "check_server  : You own pid $aa"
			}
		    } else {
			if { $mydebug == 1 } then {
			    puts "check_server  : You DON'T own pid $aa"
			}
		    }
		    #--> Just set a flag to say that there is a running sas
		    set sasrun 1
		}
	    }
	}
    }

    #--> As we have parity, now check what resources are allocated
    read_resource_settings

    if { $mydebug == 1 } then {
	puts "check_server  : localsasport = $localsasport"
	puts "check_server  : sasrun       = $sasrun"
	puts "check_server  : sasownctr    = $sasownctr"
    }

    #--> Is the localsas allocated ..?
    if { $localsasport > 0 } then {
	#--> is there a server running ?
	if { $mydebug == 1 } then { puts "check_server  : localsasport == $localsasport ( we are allocated )" }
	if { $sasrun == 1 } then {
	    #--> do we own a running port ?
	    if { $mydebug == 1 } then { puts "check_server  : sasrun == $sasrun ( Daemon running )" }
	    if { $sasownctr > 0 } then {
		#--> check that what we own and what is runing are the same
		if { $mydebug == 1 } then { puts "check_server  : sasownctr == $sasownctr ( We own a daemon )" }
		set ohmy [ lsearch $sasownlist $localsasport ]
		if { $mydebug == 1 } then { puts "check_server  : ohmy == $ohmy ( is the allocated the same as the running? )" }
		if { $ohmy == -1 } then {
		    #--> The running port is not the same as the resource so start
		    set status 3
		} else {
		    #--> All is well with the world
		    set status 4
		}
	    } else {
		#--> In this case deallocate and then create
		set status 2
	    }
	} else {
	    set status 1
	}
    } else {
	#--> no local sas allocated, in resource file
	set status 0
    }
    #--> Now Determine what to do ...
    if { $mydebug == 1 } then {
	puts "check_server  : Status = $status"
    }
    set mystatus 0
    set myctr    0
    while { $status < 4 } {
	set mystatus $status
	if { $mystatus == 0 } then {
	    #--> No localsas resource allocated
	    set status [ create_port -1 ]
	} elseif { $mystatus == 1 } then {
	    #--> resource, not running
	    set status [ create_port [ expr $localsasport - 10230 ] ]
	} elseif { $mystatus == 2 } then {
	    #--> resource, run, not owner
	    set status [ create_port -1 ]
	} elseif { $mystatus == 3 } then {
	    #--> Resource, run, own, dif between runport and resource
	    set status [ reset_resource $localsasport]
	} else {
	    #--> All Ok
	    set status 4
	}
	if { $myctr < 10 } then {
	    set myctr [ expr $myctr + 1 ]
	} else {
	    break
	}
    } 

    if { $mydebug == 1 } then { puts "check_server  : status == $status " };
    if { $mydebug == 1 } then { puts "check_server  : setting env(MIDAS_MEMSAS_PORT) to $localsasport" };
    set env(MIDAS_MEMSAS_PORT) $localsasport
}

#--> ======================================================= <--#
proc read_resource_settings {} {
    global mydebug
    global env
    global localsasport
    global resources

    #--> Ok now lets do the difficult stuff .. check the resources
    set resources {}
    set resources [ :experiment resources ]
    #--> Now look for our killer app ...  localsas
    set present    0
    set name       ""
    set capability ""
    set port       ""
    set host       ""
    set role       ""
    set type       ""
    set ip         ""

    foreach res $resources {
	foreach r $res {
	    switch [ first $r ] {
		name          { set name [ second $r ] }
		host          { set host [ second $r ] }
		role          { set role [ second $r ] }
		type          { set type [ second $r ] }
		port          { set port [ second $r ] }
		capability    { set cap  [ second $r ] }
	    }
	}
	if { $name == "localsas" } then {
	    set present 1
	    set localsasport $port
	    if { $capability != "0x0000000000000000" } then {
		set present 2
	    }
	    if { $mydebug == 1 } then {
		puts "Read_resource : Localsas found ...."
	    }
	    break;
	}
    }
    #--> Ok is the resource that we need present ?
    if { $present == 1 } then {
	#--> Present but bad capeability
	if { $mydebug == 1 } then {
	    puts "Read_resource : localsas is present but the capability is not correct"
	}
    } elseif { $present == 2 } then {
	#--> Present and all is well
	if { $mydebug == 1 } then {
	    puts "Read_resource : localsas is present and we do not need to do anything"
	}
    } else {
	set localsasport 0
    }

}
#--> +++++++++++++++++++++++++++++++++++++++++++++++++++++++ <--#
proc readpipe arg {
    set data ""
    set fileid [ open |$arg r ]
    set data   [ read $fileid ]
    close      $fileid
    return $data
}
#--> +++++++++++++++++++++++++++++++++++++++++++++++++++++++ <--#
proc create_port arg {
    global localsasport
    global serverprog
    global mydebug
    
    if { $mydebug == 1 } then {
	puts "create_port : Entered routine with argument : $arg"
    }

    if { $arg == -1 } then {
	if { $mydebug == 1 } then { puts "create_port : About to exec $serverprog -c -p-1 & " }
	set pid [ exec $serverprog -c -p-1 & ]
	if { $mydebug == 1 } then { puts "create_port : exec complete" }
	#--> Have to add 1 to pid as the daemon forks ....
	set pid [ expr $pid + 1 ]
	if { $mydebug == 1 } then {
	    puts "create_port : pid = $pid"
	}
	#--> Give the spectrum server a chance to search if need be ...
	after 2000
	set z [ catch { open "/tmp/memsas.pid.$pid" r } f ]
	if { $z != 0 } then {
	    midas-warning "Could not find /tmp/memsas.pid.$pid"
	}
	#--> Now read the localsas port from this file
	gets $f localsasport
	close $f
	if { $mydebug == 1 } then {
	    puts "create_port : localsasport is set to : $localsasport"
	}
	if { $mydebug == 1 } then {
	    puts "create_port : Exiting routine with return value of 3"
	}
	return 3
    } else {
	#--> The localsasport is specified
	if { $mydebug == 1 } then { puts "create_port : About to exec $serverprog -c -p$arg & " }
	set pid [ exec $serverprog -c -p$arg & ]
	if { $mydebug == 1 } then { puts "create_port : exec complete" }
	#--> Have to add 1 to pid as the daemon forks ....
	set pid [ expr $pid + 1 ]
	if { $mydebug == 1 } then {
	    puts "create_port : Exiting routine with return value of 4"
	}
	return 4
    }

    if { $mydebug == 1 } then {
	puts "create_port : Exiting routine!"
    }


}
#--> ======================================================= <--#
proc reset_resource arg {
    global EGallow EGRetries
    global resources mydebug localsasport env
    global afterstring
    #--> This procedure is used to reallocate resources
    #--> The first thing to do is to remove the current resource
    set directory [file join $env(MIDASTempDir) tcl[pid]]
    set name       ""
    set capability ""
    set port       ""
    set host       ""
    set role       ""
    set type       ""
    set ip         ""
    set status     0

    if { $mydebug == 1 } then { puts "reset_resource : dir = $directory" }
    if { $mydebug == 1 } then { puts "reset_resource : arg = $arg" }

    foreach res $resources {
	foreach r $res {
	    switch [ first $r ] {
		name          { set name [ second $r ] }
		host          { set host [ second $r ] }
		role          { set role [ second $r ] }
		type          { set type [ second $r ] }
		port          { set port [ second $r ] }
		capability    { set cap  [ second $r ] }
	    }
	    if { $mydebug == 1 } then {
		puts $r
	    }
	}
	if { $name == "localsas" } then {
	    set status 1
	    break;
	}
	if { $mydebug == 1 } then {
	    puts $name
	}
    }

    if { $status == 1 } then {
	#--> Resource present .....
	if { $mydebug == 1 } then { puts "reset_resource : localsas present (res)" }
	set z [ catch { open "localsas" w } f ] 
	if { $z == 0 } then {
	    puts $f "((name $name)"
	    puts $f " (host $host)"
	    puts $f " (role $role)"
	    puts $f " (type $type)"
	    puts $f " (ipaddress 123456)"
	    puts $f " (port $port))"
	    close $f
	    
	    :apparatus free   :name "localsas"
	    :apparatus detach :name "localsas"

	} else {
	    midas-warning "Could not create localsas"
	}
    } 
    #--> Now let us generate, claim and attach the correct resource
    set z [ catch { open "localsas" w } f ] 
    set name "localsas"
    set host "localhost"
    set role "localsas"
    set type "spectrum"
    set port $arg

    if { $mydebug == 1 } then { 
	puts "reset_resource : z    = $z" 
	puts "reset_resource : name = $name" 
	puts "reset_resource : host = $host" 
	puts "reset_resource : role = $role" 
	puts "reset_resource : type = $type" 
	puts "reset_resource : port = $port" 
    }

    if { $z == 0 } then {
	puts $f "((name $name)"
	puts $f " (host $host)"
	puts $f " (role $role)"
	puts $f " (type $type)"
	puts $f " (ipaddress 123456)"
	puts $f " (port $port))"
	close $f
	

	setup_alloc $directory

    }
   
    if { $mydebug == 1 } then { puts "reset_resource : returning 4 " }

    return 4
}

proc setup_alloc arg {
    global mydebug
    if { $mydebug == 1 } then { puts "setup_alloc: Entered" }
    :apparatus attach :path $arg :name "localsas"
    :apparatus claim  :name "localsas"
    if { $mydebug == 1 } then { puts "setup_alloc: Departing" }

}
