

#   clover.defn.tcl     Eurogam II Clover detector VXI card


proc {AddCloverModule}  {module slot}  {
   global registerbase cardtype

   set registerbase $cardtype$module

   delete-vxi-registers $registerbase.*

   CloverVXIRegisters $slot
   CloverModuleRegisters  $slot

   CloverGeRegisters $module $slot
   CloverBGORegisters $module $slot
}

proc CloverBGORegisters {module slot} {
   global registerbase cardtype

   set registerbase $cardtype$module.BGO
   CloverVXIRegisters $slot
   CloverModuleRegisters  $slot

   CloverBGODetectorRegisters  $slot
}

proc CloverGeRegisters {module slot} {
   global registerbase cardtype

   set registerbase $cardtype$module.Ge
   CloverVXIRegisters $slot
   CloverModuleRegisters  $slot

   CloverGeModuleRegisters  $slot

   set seg 1
   foreach segment {A B C D E} {
      set registerbase $cardtype$module.Ge$segment
      CloverGeDetectorRegisters  $slot $seg
      incr seg
   }
}

proc clover_register {class registername attributes} {
   global registerbase
   define-vxi-register $registerbase $class $registername "$attributes"
}

proc CloverVXIRegisters {slot} {
   clover_register xSht     VXI.ManID      "-s$slot -o0  -pro -w16 -fx -b0  -l12"
   clover_register xSht     VXI.AddrSpace  "-s$slot -o0  -pro -w16 -fd -b12 -l2"
   clover_register xSht     VXI.DevClass   "-s$slot -o0  -pro -w16 -fd -b14 -l2"
   clover_register xSht     VXI.ModelCode  "-s$slot -o2  -pro -w16 -fx -b0  -l12"
   clover_register xSht     VXI.ReqMem     "-s$slot -o2  -pro -w16 -fd -b12 -l4"
   clover_register xSht     VXI.Reset      "-s$slot -o4  -prw -w16 -fd -b0  -l1"
   clover_register xSht     VXI.Passed     "-s$slot -o4  -pro -w16 -fd -b2  -l1"
   clover_register xSht     VXI.Ready      "-s$slot -o4  -pro -w16 -fd -b3  -l1"
   clover_register xSht     VXI.ModID      "-s$slot -o4  -pro -w16 -fd -b14 -l1"
   clover_register xSht     VXI.A24A32     "-s$slot -o4  -pro -w16 -fd -b15 -l1"
   clover_register xSht     VXI.Offset     "-s$slot -o6  -pro -w16 -fx"
   clover_register xSht     VXI.SerialNum  "-s$slot -o8  -pro -w16 -fx"
   clover_register xSht     VXI.ModLevel   "-s$slot -o10 -pro -w16 -fx"
}

proc CloverModuleRegisters {slot} {
   global registerbase

   clover_register  xDig  LastCard       "-s$slot -c0 -o0x38 -w16 -prw -fd -b0 -l1 -i0"
   clover_register  xDig  PileupReject   "-s$slot -c0 -o0x38 -w16 -prw -fd -b1  -l1 -i1"
   clover_register  xDig  FIFO.Reset     "-s$slot -c0 -o0x38 -w16 -prw -fd -b3  -l1 -i0"
   clover_register  xDig  ConFenetres    "-s$slot -c0 -o0x38 -w16 -prw -fd -b6  -l1 -i0"
   clover_register  xDig  TestGen.Attn   "-s$slot -c0 -o0x38 -w16 -prw -fd -b7  -l1 -i0"
   clover_register  xDig  FIFO.NotEmpty  "-s$slot -c0 -o0x38 -w16 -pro -fd -b15 -l1"
   clover_register  xFIT  FIFO.Test      "-s$slot -c0 -o0x3c -w32 -prw -fx"
   clover_register  xMty  FIFO.Flush     "-s$slot -o0x3c -e0x38 -b15"

#    Read Out registers

   clover_register  xDig RdOut.EvNum.Count "-s$slot -c0 -o0x40 -w32 -prw -fx -b0 -l0 -i0"
   clover_register  xAdr RdOut.EvNum.Addr  "-s$slot -c0 -o0x44 -w32 -b0 -l0 -i$registerbase.RdOut.EvNum"
   clover_register  xDig RdOut.ROCR        "-s$slot -c0 -o0x48 -w32 -prw -fx -b0 -l0 -i0"
   clover_register  xDig RdOut.Control     "-s$slot -c0 -o0x48 -w32 -prw -fx -b0 -l0"
   clover_register  xDig RdOut.Status      "-s$slot -c0 -o0x48 -w32 -prw -fx -b0 -l0"
   clover_register  xDig RdOut.EventFIFO   "-s$slot -o0x4c -w32 -prw -fx -b0 -l0"
   clover_register  xDig RdOut.Inspection  "-s$slot -o0x60 -w32 -prw -fx -b0 -l0"
   clover_register  xDig RdOut.FIFOTest    "-s$slot -o0x7c -w32 -prw -fx -b0 -l0"
   clover_register  xDig RdOut.ROCR.ReadOutMode       "-s$slot -o0x48 -w32 -prw -fx -b0 -l1 -i0"
   clover_register  xDig RdOut.ROCR.EvNumQbits.Enable "-s$slot -o0x48 -w32 -prw -fx -b1 -l1 -i1"
   clover_register  xDig RdOut.ROCR.PipeLine.Enable   "-s$slot -o0x48 -w32 -prw -fx -b2 -l1"
   clover_register  xDig RdOut.ROCR.EvNumData.Enable  "-s$slot -o0x48 -w32 -prw -fx -b3 -l1"
   clover_register  xDig RdOut.ROCR.EvFault1          "-s$slot -o0x48 -w32 -prw -fx -b8 -l1"
   clover_register  xDig RdOut.ROCR.EvFault2          "-s$slot -o0x48 -w32 -prw -fx -b9 -l1"
   clover_register  xDig RdOut.ROCR.FIFO1empty        "-s$slot -o0x48 -w32 -prw -fx -b10 -l1"
   clover_register  xDig RdOut.ROCR.FIFO2empty        "-s$slot -o0x48 -w32 -prw -fx -b11 -l1"
   clover_register  xDig RdOut.ROCR.FIFO3empty        "-s$slot -o0x48 -w32 -prw -fx -b12 -l1"
   clover_register  xDig RdOut.ROCR.FIFO1full         "-s$slot -o0x48 -w32 -prw -fx -b13 -l1"
   clover_register  xDig RdOut.ROCR.FIFO2full         "-s$slot -o0x48 -w32 -prw -fx -b14 -l1"
   clover_register  xDig RdOut.ROCR.FIFO3full         "-s$slot -o0x48 -w32 -prw -fx -b15 -l1"
}

#    board common germanium registers
proc CloverGeModuleRegisters {slot} {
   clover_register  xDAC CFD.Width  "-s$slot -c0 -o0x00 -w16 -pwo -qn -us  -d0 -r0:0s,0xff:200ns -i50ns"
   clover_register  xDAC FTSample   "-s$slot -c0 -o0x02 -w16 -pwo -qn -us  -d0 -r0:0s,0xff:3.2us -i515ns"
   clover_register  xDAC ValSample  "-s$slot -c0 -o0x04 -w16 -pwo -qn -us  -d0 -r0:0s,0xff:16us  -i2.51us"
   clover_register  xDAC Watchdog   "-s$slot -c0 -o0x06 -w16 -pwo -qn -us  -d0 -r0:0s,0xff:16us  -i15.25us"
   clover_register  xRng TestGen    "-s$slot -c0 -o0x08 -g0x3a -b0 -qk -ueV -d0 -r0:0eV,0xffff:7MeV -R0:0eV,0xffff:3MeV -I0 -i4keV"
}

#    germanium detector registers
proc CloverGeDetectorRegisters {slot seg} {
  global registerbase

   clover_register  xMux Insp.A1    "-s$slot -c$seg -o0x30 -b0 -l14 -dClover -ma1"
   clover_register  xMux Insp.A2    "-s$slot -c$seg -o0x30 -b3 -l13 -dClover -ma2"
   clover_register  xMux Insp.D1    "-s$slot -c$seg -o0x32 -b0 -l8 -dClover -md1"
   clover_register  xMux Insp.D2    "-s$slot -c$seg -o0x32 -b8 -l8 -dClover -md2"
   clover_register  xMux Insp.V     "-s$slot -c$seg -o0x34 -b0 -l8 -dClover -mv"
   clover_register  xRng CFD.Thresh     "-s$slot -c$seg -o0x00 -g0x20 -b2 -qk -ueV -d0 -r0:0eV,0xff:2.6MeV -R0:0eV,0xff:825keV -I1 -i50keV"
   clover_register  xDAC CFD.Delay      "-s$slot -c$seg -o0x02 -w16 -pwo -qn -us -d0 -r0:0s,0xff:200ns -i100ns"
   clover_register  xDig PoleZero       "-s$slot -c$seg -o0x04 -w16 -pwo -fd -l8 -i200"
   clover_register  xDAC PDSGate        "-s$slot -c$seg -o0x06 -w16 -pwo -qn -us -d0 -r0:0s,0xff:16us  -i4.016us"
   clover_register  xDig TFAGain        "-s$slot -c$seg -o0x20 -w16 -prw -fd -b2 -l1 -i1"
   clover_register  xDig TestMode       "-s$slot -c$seg -o0x20 -w16 -prw -fd -b4 -l1 -i0"
   clover_register  xDig Enable         "-s$slot -c$seg -o0x20 -w16 -prw -fd -b5 -l1 -i0"
   clover_register  xDig 20MeV.Thresh   "-s$slot -c$seg -o0x40 -w32 -pwo -fd -b16 -l16 -i0"
   clover_register  xDig 4MeV.Thresh    "-s$slot -c$seg -o0x44 -w32 -pwo -fd -b16 -l16 -i0"
   clover_register  xDig FT.Thresh      "-s$slot -c$seg -o0x48 -w32 -pwo -fd -b16 -l16 -i0"
   clover_register  xDig CO.Thresh      "-s$slot -c$seg -o0x4c -w32 -pwo -fd -b16 -l16 -i0"
   clover_register  xAdr 20MeV.Addr     "-s$slot -c$seg -o0x50 -w32 -i$registerbase.20MeV"
   clover_register  xAdr 4MeV.Addr      "-s$slot -c$seg -o0x54 -w32 -i$registerbase.4MeV"
   clover_register  xAdr FT.Addr        "-s$slot -c$seg -o0x58 -w32 -i$registerbase.FT"
   clover_register  xAdr CO.Addr        "-s$slot -c$seg -o0x5c -w32 -i$registerbase.CO"
   clover_register  xDig 20MeV.TestRd   "-s$slot -c$seg -o0x60 -w32 -pro -fx"
   clover_register  xDig 4MeV.TestRd    "-s$slot -c$seg -o0x64 -w32 -pro -fx"
   clover_register  xDig FT.TestRd      "-s$slot -c$seg -o0x68 -w32 -pro -fx"
   clover_register  xDig CO.TestRd      "-s$slot -c$seg -o0x6c -w32 -pro -fx"
   clover_register  xDig 20MeV.SSEnable "-s$slot -c$seg -o0x70 -w32 -prw -fd -b16 -l1 -i1"
   clover_register  xDig 4MeV.SSEnable  "-s$slot -c$seg -o0x70 -w32 -prw -fd -b17 -l1 -i1"
   clover_register  xDig FT.SSEnable    "-s$slot -c$seg -o0x70 -w32 -prw -fd -b18 -l1 -i1"
   clover_register  xDig CO.SSEnable    "-s$slot -c$seg -o0x70 -w32 -prw -fd -b19 -l1 -i1"
   clover_register  xDig 20MeV.ROEnable "-s$slot -c$seg -o0x74 -w32 -prw -fd -b16 -l1 -i1"
   clover_register  xDig 4MeV.ROEnable  "-s$slot -c$seg -o0x74 -w32 -prw -fd -b17 -l1 -i1"
   clover_register  xDig FT.ROEnable    "-s$slot -c$seg -o0x74 -w32 -prw -fd -b18 -l1 -i1"
   clover_register  xDig CO.ROEnable    "-s$slot -c$seg -o0x74 -w32 -prw -fd -b19 -l1 -i1"
   clover_register  xDig SlidingScale   "-s$slot -c$seg -o0x78 -w32 -pro -fd -b16 -l16"
   clover_register  xDig ROCITest       "-s$slot -c$seg -o0x7c -w32 -pwo -fd -b16 -l16 -i0"
}


#    bgo detector registers
proc CloverBGODetectorRegisters {slot} {
   global registerbase

   clover_register  xDAC DT.Width   "-s$slot -c7 -o0x02 -w16 -pwo -qn -us -d0 -r0:2us,0xff:20us  -i4.8us"
   clover_register  xDAC FTSample   "-s$slot -c7 -o0x06 -w16 -pwo -qn -us -d0 -r0:0s,0xff:3.2us  -i640ns"
   clover_register  xDAC ValSample  "-s$slot -c7 -o0x08 -w16 -pwo -qn -us -d0 -r0:0s,0xff:16us   -i2.5us"
   clover_register  xDAC Watchdog   "-s$slot -c7 -o0x0a -w16 -pwo -qn -us -d0 -r0:0s,0xff:16us   -i9.8us"
   clover_register  xDAC PDSGate    "-s$slot -c7 -o0x0c -w16 -pwo -qn -us -d0 -r0:0s,0xff:16us   -i5.083us"
   clover_register  xMux Insp.A1    "-s$slot -c6 -o0x30 -b0 -l14 -dClover -ma1"
   clover_register  xMux Insp.A2    "-s$slot -c6 -o0x30 -b3 -l13 -dClover -ma2"
   clover_register  xMux Insp.D1    "-s$slot -c6 -o0x32 -b0 -l8 -dClover -md1"
   clover_register  xMux Insp.D2    "-s$slot -c6 -o0x32 -b8 -l8 -dClover -md2"
   clover_register  xMux Insp.V     "-s$slot -c6 -o0x34 -b0 -l8 -dClover -mv"
   clover_register  xDAC Disc.Thresh        "-s$slot -c6 -o0x00 -w16 -pwo -qk -ueV -d0 -r0:88.8keV,0x7ff:0eV -i48keV"
   clover_register  xDAC TestGen            "-s$slot -c6 -o0x02 -w16 -pwo -qm -uV  -d0 -r0:0V,0xfff:5V -i3mV"
   clover_register  xDig a.TestMode.Disable "-s$slot -c6 -o0x20 -w16 -prw -fd -b0 -l1 -i1"
   clover_register  xDig b.TestMode.Disable "-s$slot -c6 -o0x20 -w16 -prw -fd -b1 -l1 -i1"
   clover_register  xDig c.TestMode.Disable "-s$slot -c6 -o0x20 -w16 -prw -fd -b2 -l1 -i1"
   clover_register  xDig d.TestMode.Disable "-s$slot -c6 -o0x20 -w16 -prw -fd -b3 -l1 -i1"
   clover_register  xDig e.TestMode.Disable "-s$slot -c6 -o0x20 -w16 -prw -fd -b4 -l1 -i1"
   clover_register  xDig f.TestMode.Disable "-s$slot -c6 -o0x20 -w16 -prw -fd -b5 -l1 -i1"
   clover_register  xDig g.TestMode.Disable "-s$slot -c6 -o0x20 -w16 -prw -fd -b6 -l1 -i1"
   clover_register  xDig h.TestMode.Disable "-s$slot -c6 -o0x20 -w16 -prw -fd -b7 -l1 -i1"
   clover_register  xDig Disable            "-s$slot -c6 -o0x20 -w16 -prw -fd -b10 -l1 -i1"
   clover_register  xDig Pattern.Thresh     "-s$slot -c6 -o0x40 -w32 -prw -fd -b16 -l16 -i0"
   clover_register  xDig TAC.Thresh         "-s$slot -c6 -o0x44 -w32 -prw -fd -b16 -l16 -i0"
   clover_register  xDig Energy.Thresh      "-s$slot -c6 -o0x48 -w32 -prw -fd -b16 -l16 -i0"
   clover_register  xAdr Pattern.Addr       "-s$slot -c6 -o0x50 -w32 -i$registerbase.Pattern"
   clover_register  xAdr TAC.Addr           "-s$slot -c6 -o0x54 -w32 -i$registerbase.TAC"
   clover_register  xAdr Energy.Addr        "-s$slot -c6 -o0x58 -w32 -i$registerbase.Energy"
   clover_register  xDig Pattern.TestRd     "-s$slot -c6 -o0x60 -w32 -pro -fx"
   clover_register  xDig TAC.TestRd         "-s$slot -c6 -o0x64 -w32 -pro -fx"
   clover_register  xDig Energy.TestRd      "-s$slot -c6 -o0x68 -w32 -pro -fx"
   clover_register  xDig Pattern.SSEnable   "-s$slot -c6 -o0x70 -w32 -prw -fd -b16 -l1 -i0"
   clover_register  xDig TAC.SSEnable       "-s$slot -c6 -o0x70 -w32 -prw -fd -b17 -l1 -i1"
   clover_register  xDig Energy.SSEnable    "-s$slot -c6 -o0x70 -w32 -prw -fd -b18 -l1 -i1"
   clover_register  xDig Pattern.ROEnable   "-s$slot -c6 -o0x74 -w32 -prw -fd -b16 -l1 -i1"
   clover_register  xDig TAC.ROEnable       "-s$slot -c6 -o0x74 -w32 -prw -fd -b17 -l1 -i1"
   clover_register  xDig Energy.ROEnable    "-s$slot -c6 -o0x74 -w32 -prw -fd -b18 -l1 -i1"
   clover_register  xDig SlidingScale       "-s$slot -c6 -o0x78 -w32 -pro -fd -b16 -l16"
   clover_register  xDig ROCITest           "-s$slot -c6 -o0x7c -w32 -pwo -fd -b16 -l16 -i0"
   clover_register  xSup SuppressionTable   "-s$slot -iSimple"
}


proc Clover_DAQ-reset   {args}   {
   set module [first $args]
#    flush FIFOs
   write-vxi-register $module.FIFO.Reset 0
   write-vxi-register $module.FIFO.Reset 1
   write-vxi-register $module.FIFO.Reset 0
}


proc Clover_DAQ-halt   {args}   {
   set module [first $args]
#    no action
}


proc Clover_DAQ-go   {args}   {
   set module [first $args]
#    flush FIFOs
   write-vxi-register $module.FIFO.Reset 0
   write-vxi-register $module.FIFO.Reset 1
   write-vxi-register $module.FIFO.Reset 0
#    zero event numbers
   write-vxi-register $module.RdOut.EvNum.Count 0
}
