#    hist task code overlay for detector type "tapered"

set lowchannel 0
set highchannel 35
set maxchannel 35
set qualifier 1

set genames  {20MeV 4MeV FT CO}
set bgonames {Energy TAC Pattern}
set qnames   {q0 q1 q2 q3}
set qualnames {0 1 2 3 0+2}

set germanium 0
set bgo 0


proc make-overlaypanel {} {
   global genames bgonames qnames

      :layout V; :row-gap 10; :col-gap 0

      :message DET-M1  :label "Tapered detector overlay applied"

      :number DET-LOW  :label "Low channel:"  :w 5 :valuex 15c
      :number DET-HIGH :label "High channel:" :w 5 :valuex 15c

      :choice DET-QUAL :label "Qualifier"  :valuex 15c :exclusive F :strings $qnames   :value 1
      :choice DET-GE   :label "ADC - Ge:"  :valuex 15c :exclusive F :strings $genames  :value 0
      :choice DET-BGO  :label "ADC - BGO:" :valuex 15c :exclusive F :strings $bgonames :value 0
}


#    this proc handles events from widgets created in the proc make-overlaypanel
#    The widgets DET-LOW; DET-HIGH and DET-QUAL are handled in the main click procedure
#    and are not required here

proc click-overlay {w v} {

   switch -glob $w {
      DET-GE           {click-adc $v germanium}
      DET-BGO          {click-adc $v bgo}
      default      {puts stdout $arg}
   }
}

proc repaint_overlay {} {
   global spectrumlist
   global germanium bgo qualifier

   if {[llength $spectrumlist] > 0} then {
       :modify DET-GE  :value 0
       :modify DET-BGO :value 0
       :modify DET-QUAL :value 0
   } else {
       :modify DET-GE  :value $germanium
       :modify DET-BGO :value $bgo
       :modify DET-QUAL :value $qualifier
   }
}


proc get-spectrum-list {} {
        global spectrumlist
        global genames bgonames germanium bgo
        global qualifier qualnames
        global lowchannel highchannel

        if {$spectrumlist != {}} then {return $spectrumlist};  # return names selected from spectrum list

        set qname02 "0+2"

        set qlist [list-take $qualnames [bits-in $qualifier]]
        set qplace02 [lsearch $qlist $qname02]
        set list {}

        for {set i $lowchannel} {$i<=$highchannel} {incr i} {
# append ge spectrum names
           foreach adc [list-take $genames [bits-in $germanium]] {
              foreach q $qlist {
                  if {$q=="1" || $q=="3"} then {lappend list Ge$i.$adc.q$q}
                  if {$adc=="FT" || $adc=="CO"} then {
                     if {$q=="0" || $q=="2" || $q=="0+2"} then {lappend list Ge$i.$adc.q$q}
                  } else {
                     if {$q=="0+2" || \
                         (($q=="0" || $q=="2")  &&  $qplace02==-1)} then {lappend list Ge$i.$adc.q$q}
                  }
               }
            }
# and append corresponding bgo spectrum names
            foreach adc [list-take $bgonames [bits-in $bgo]] {lappend list BGO$i.$adc}
        }
       return $list
}

proc range-of {n} {

        set fields [split $n "."]
        if {[string match "Pattern" [second $fields]]} then {return 0x800};  # BGO Pattern item
        return 0x2000;  #  everything else
}
