# paril.tcl   

#    The array "table" which is indexed by the "widget name" contains all the information needed
#    for the register/display item
#
#    Each element of "table" is a list
#
#       0.    name of global parameter used to hold current value of the register
#       1.    procedure to be used to convert the widget value into parameter value
#       2.    procedure to be used to convert the parameter value into a widget value
#       3.    name of the register
#       4.    procedure to be used to convert the register value into parameter value
#       5.    procedure to be used to convert the parameter value into a register value
#       6.    name of unit used for register access
#       7.    widget type to be used for "by parameter" displays
#       8-10  parameters for the procedure creating "by parameter" displays
#              eg  if (7) is :slider then 8 is min value, 9 is max value and 10 is unit
#       11    for startup only - initial value of the parameter (1)
#       12    for startup only - inserted for this widget into the table "parameternames"
global chansel chanlist chanindex



set table(PRCS0) {prcs0 widget->param param->hex-widget "PARIL.RCS.SingleEnable"
       int-reg->param param->int-reg nil :text 6 nil nil 1 "RCS"}
set table(PRCS1) {prcs1 widget->param param->hex-widget "PARIL.RCS.TAC_TFG"
       int-reg->param param->int-reg nil :text 6 nil nil 1 "RCS"}
set table(PRCS2) {prcs2 widget->param param->hex-widget "PARIL.RCS.TFGEnable"
       int-reg->param param->int-reg nil :text 6 nil nil 1 "RCS"}
set table(PRCS3) {prcs3 widget->param param->hex-widget "PARIL.RCS.LinkAckFP_AP"
       int-reg->param param->int-reg nil :text 6 nil nil 1 "RCS"}
set table(PRCS4) {prcs4 widget->param param->hex-widget "PARIL.RCS.LTreset"
       int-reg->param param->int-reg nil :text 6 nil nil 1 "RCS"}

set table(PRCR) {prcr str-widget->param param->hex-widget "DPPC.ROEnable"
       int-reg->param param->int-reg nil :text 6 nil nil 1 ""}

set table(PRTFGCOUNT) {prtfgcount widget->param param->hex-widget "PARIL.TFG_Count"
       int-reg->param param->int-reg nil :text 6 nil nil 1 "Tfg count"}
set table(PRTFGSTEP) {prtfgstep widget->param param->hex-widget "PARIL.TFG_Step"
       int-reg->param param->int-reg nil :text 6 nil nil 1 "Tfg count"}
set table(PRSCALER) {prscaler str-widget->param param->int-widget "PARIL.Scaler"
       int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(PRREALTFGCOUNT) {prrealtfgcount str-widget->param param->hex-widget "PARIL.RealTFG_Count"
       int-reg->param param->int-reg nil nil nil nil nil 1 nil}

#set table(STARTRDTSINGLE) {startrdtsingle int-widget->param param->int-widget "PARIL.SSRS_Cmd"
#       int-reg->param param->int-reg nil :button nil nil nil 1 "Start Readout"}
#set table(STARTRDTCO) {startrdtco int-widget->param param->int-widget "PARIL.SSRC_Cmd"
#       int-reg->param param->int-reg nil :button nil nil nil 1 "Start Readout"}

set insp_table(MUX-PAR1) {"PARIL.Insp1" PAR}
set insp_table(MUX-PAR2) {"PARIL.Insp2" PAR}
set insp_menu(MUX-PAR1) parilinsptable
set insp_menu(MUX-PAR2) parilinsptable
set insp_widgetnames {MUX-PAR1 MUX-PAR2}
set insp_regnames    {PARIL.Insp1 PARIL.Insp2}

set parilinsptable {Disconnected
      Encod Sing-Ack Read-Convert Busy EOC DVD CDR DPPC-IAL DPPC-CS DPPC-WR DPPC-RD IACK DPPC-DIN Corr-Add-Run WS-RAM CS-RAM End-DPPC-Rdt WS-6Mev WS-TAC CS-Singles Prepileup Postpileup Veto StartRdt_Singles Latch_Scaler LinkAck OeLinkBusEvt LinkBus0 LinkClk CountTFG ClearTFG}

set PAR-Param      {* 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31}

 
proc start-PARIL {} {
#  display PARIL sub frame
   global parilvisible parilframe
   if {$parilvisible==0} then {make-PARIL} else {open-frame $parilframe}
   click-redisplay
} 

proc click-startrdtsingle {args} {write-vxi-reg PARIL.SSRS_Cmd 1}
proc click-startrdtco     {args} {write-vxi-reg PARIL.SSRC_Cmd 1}

set parilframe [self frame-name].paril
set parilvisible 0

proc make-PARIL {} {
  global env parilvisible parilframe parilinsptable

set col1 40c

  :frame $parilframe :label "HRGe8x PARIL" :show-footer T :resize F :transient T 


   :panel P1 :layout T :padx 10
     :layout V; :col-gap 5;  :row-gap 5

      :text     PARILCHANNEL    :label "Current Channel"  :deaf T :valuex 20c :w 3
      :message  PRCSM     :label "Singles Control Register (PRCS): " :bold T
      :checkbox PRCS0      :label "Enable Singles Readout" :valuex 20c
      :choice   PRCS1      :label "Time Reference" :strings [list TAC TFG] :valuex 20c
      :checkbox PRCS2      :label "Enable TFG to count" :valuex 20c
      :choice   PRCS3      :label "Connect Link_Ack to" :strings [list "Front Panel" EVRI] :valuex 20c
      :checkbox PRCS4      :label "Enable LT Reset" :valuex 20c
      :next-row T
      :message  PRCRM    :label "Corrolated Control Register (PRCR): " :bold T
      :text PRCR         :label "DPPC Event Readout" :w 6 :valuex 20c
      :next-row T
      :message  PRSM     :label  "PARIL Setup" :bold T
      :text PRTFGCOUNT   :label "TFG Counter" :w 8 :valuex 20c
      :text PRTFGSTEP    :label "TFG Step" :w 8 :valuex 20c
      :text PRSCALER     :label "Scaler:"  :deaf T :valuex 20c :w 6
      :text PRREALTFGCOUNT    :label "Real TFG Count:"  :deaf T :valuex 20c :w 6

      :button STARTRDTSINGLE :label "Simulate Start Readout Singles Command"     :command click-startrdtsingle
      :button STARTRDTCO     :label "Simulate Start Readout Corrolated Command"  :command click-startrdtco

      :layout H
      :next-row T
      :button ALLSCALER      :label "Read scalers"  :command click-allscaler 
      :button RESETALLSCALER :label "Reset scalers" :command click-resetallscaler 

   :panel P4 :layout T :padx 10 :pady 10
      :layout H; :col-gap 10; :row-gap 10

      :message INSPECTION   :label "Inspection Lines: " :bold T
      :next-row T
      :menubutton MUX-PAR1 :label "Paril Inspection 1:" :w 18 :strings $parilinsptable :rows 10
      :message MUX-PAR1-USE   :label ""
      :next-row T
      :menubutton MUX-PAR2 :label "Paril Inspection 2:" :w 18 :strings $parilinsptable :rows 10
      :message MUX-PAR2-USE   :label ""

   :panel BUTTONS :layout B :padx 10 :after FOOTER
      :layout H
      :button PARILREDISPLAY :label Redisplay
      :hskip 150
      :button HELP

  :show T
  set parilvisible 1
}
 ###################################

proc click-resetallscaler {args} {
   global  chanlist seglist chanindex segindex cardtype 


   for {set i 0} {$i<=1} {incr i} {
   for {set j 0} {$j<=3} {incr j} {
	set c [lindex $chanlist $i]
        set chanindex $i
        set s [lindex $seglist $j]
        set segindex $j
         catch {:modify CHANNEL :value $i}
         catch {:modify SEGMENT :value $j}


# "" remplace le parametre slot number "
# $c est le numero de voie
# 1 est la value
# 0 ? mais il faut le mettre :-)

write-reg $c $s PARIL.RCS.LTreset 1 0
write-reg $c $s PARIL.RCS.LTreset 0 0
}}

set currentcardtype $cardtype
set  cardtype RM
    write-common-reg 1 MemEch 1 0
    write-common-reg 1 MemEch 0 0
set cardtype $currentcardtype

   for {set i 0} {$i<=1} {incr i} {
   for {set j 0} {$j<=3} {incr j} {
	set c [lindex $chanlist $i]
        set chanindex $i
        set s [lindex $seglist $j]
        set segindex $j
         catch {:modify CHANNEL :value $i}
         catch {:modify SEGMENT :value $j}

# "" remplace le parametre slot number "
# $c est le numero de voie

   set datareg [read-reg $c $s PARIL.Scaler]
insert-log "[format " scaler%d%s=%d " $c $s $datareg ]"
}}


}

 ###################################
proc click-allscaler {args} {
   global  chanlist seglist chanindex segindex cardtype 

set currentcardtype $cardtype
set  cardtype RM

# 1 est le numero du RM
# 1 est la value
# 0 ? mais il faut le mettre :-)

    write-common-reg 1 MemEch 1 0
    write-common-reg 1 MemEch 0 0
set cardtype $currentcardtype


   for {set i 0} {$i<=1} {incr i} {
   for {set j 0} {$j<=3} {incr j} {
	set c [lindex $chanlist $i]
        set chanindex $i
        set s [lindex $seglist $j]
        set segindex $j
         catch {:modify CHANNEL :value $i}
         catch {:modify SEGMENT :value $j}

   set datareg [read-reg $c $s PARIL.Scaler]
insert-log "[format " scaler%d%s=%d " $c $s $datareg ]"

}}

}
