
set frame RMLADISP
set title "RM Logic Analyser Display"
set icon RMLAdisp


if {[info exists env(VXIServer)] == 0} then {set resource VXI}  \
   else {set resource $env(VXIServer)}

set chanlist [inform rmlactl set chanlist]
set chanindex [inform rmlactl set chanindex]
set RMchan [lindex $chanlist $chanindex]

set trace_index [ list 6 4 2 3 7 1 5 0 ]

# This list is position of the trace in the window
 
set tracetitles {"FT"
 		"Val3"
 		"DInsp1"
 		"DInsp2"
 		"Ext"
 		"InhReq"
 		"InhAct"
 		"Val1" }

set colours [list white white magenta yellow green]

for {set i 0} {$i < 8} {incr i} {set trace($i) {8192}}

set height 50
set xrange 512
set yrange 400

set xbase 8

set xmin 0
set xmax 8192
set xscale [expr [expr $xmax - $xmin]/$xrange]
set zoomrange [expr [expr $xmax - $xmin] / 4]

set cursor(1) $xbase
set cursor(2) $xbase
set cursor(3) $xbase
set cursor(4) $xbase

# cursor 4 is the trigger position ! patrick.

set place 8
set trigger_position	[expr $place * 512 ]

# this is the trigger point and place will be 8 by default.

proc make-frame {} {
  global env frame title icon trigger_position
  global tracetitles colours
  global xrange yrange xbase
  global height
    
   :frame $frame :label "$title" :icon-name "$icon" :show-footer T :resize F
   :panel P1 :layout T :padx 5 :pady 5

   :canvas TRACE  :w [expr $xrange + 10] :h [expr $yrange + 20]
     [:path TRACE] configure -background black

   for {set i 0} {$i < 8} {incr i} {
     do_trace $i
     :draw text :tag l$i :delete T :colour [first $colours] :text "[lindex $tracetitles $i]" \
        :value 20  [expr [expr [expr $i + 1] * $height] - 35]
   }

   for {set i 1} {$i <= 3} {incr i} {
     bind [:path TRACE] <Control-ButtonPress-$i>   "activate $i %x"
     bind [:path TRACE] <ButtonRelease-$i> "deactivate $i"
     make_cursor $i $xbase
     show_posn $i
   }
   make_cursor 4 $trigger_position
#   show_posn 4	
   bind [:path TRACE] <Double-ButtonPress-1> "zoom in"
   bind [:path TRACE] <Shift-Double-ButtonPress-1> "zoom out"
   bind [:path TRACE] <Double-ButtonPress-2> "expand 2"
   bind [:path TRACE] <Shift-Double-ButtonPress-2> "shift l"
   bind [:path TRACE] <Double-ButtonPress-3> "expand 3"
   bind [:path TRACE] <Shift-Double-ButtonPress-3> "shift r"

   :panel BUTTONS :layout B
   :layout H; :col-gap 20
   :button REDISPLAY :label "Update display"
   :button RESET     :label "Reset display"
   :button HARDCOPY  :label "Hardcopy"
   :button HELP      :command click-help
   :col-gap 5
   :hskip 100
   :button ZOOMIN    :label "@$env(ICONHOME)/expand"
   :button ZOOMOUT   :label "@$env(ICONHOME)/contract"
   :button LEFT      :label "@$env(ICONHOME)/leftarrow"
   :button RIGHT     :label "@$env(ICONHOME)/rightarrow"

   :show T
}

proc make_cursor {n x} {
   global colours cursor yrange xbase xscale xmin
   set cursor($n) [expr [expr [expr $x - $xbase] * $xscale] + $xmin]
   :draw line :tag c$n :delete T :colour [lindex $colours $n] :value $x 0 $x $yrange
}

proc do_cursor {n} {
   global colours cursor yrange xbase xscale xmin
   set x [expr [expr [expr $cursor($n) - $xmin] / $xscale] + $xbase]
   :draw line :tag c$n :delete T :colour [lindex $colours $n] :value $x 0 $x $yrange
}

proc activate   {n x} {
   global cursor xbase xscale xmin
   set cursor($n) [expr [expr [expr $x - $xbase] * $xscale] + $xmin]
   bind [:path TRACE] <Motion>  "make_cursor $n %x"
   make_cursor $n $x
}

proc deactivate {n} {
   bind [:path TRACE] <Motion> ""
   show_posn $n
}

proc expand {n} {
   global xmin xmax cursor
   set xmin $cursor(1)
   set xmax $cursor($n)
   adjustxlims
   do-display
}

proc zoom {d} {
   global xmin xmax zoomrange cursor
   if {$d == "in"} then {
      set zoomrange [expr [expr $xmax - $xmin] / 4]
   } else {
      set zoomrange [expr $xmax - $xmin]
   }
   set xmin [expr $cursor(1) - $zoomrange]
   if {$xmin < 0} then {set xmin 0}
   set xmax [expr $cursor(1) + $zoomrange]
   if {$xmax > 8192} then {set xmax 8192}
   adjustxlims
   do-display
}

proc click-zoomin args {zoom in}
proc click-zoomout args {zoom out}

proc shift {d} {
   global xmin xmax xrange
   set r [expr [expr $xmax - $xmin] / 2]
   if {$d == "r"} then {
      set xmin [expr $xmin + $r]
      set xmax [expr $xmax + $r]
   } else {
      set xmin [expr $xmin - $r]
      set xmax [expr $xmax - $r]
   }
   if {$xmin < 0}    then {set xmin 0}
   if {$xmin > [expr 8192 - $xrange]} then {set xmin [expr 8192 - $xrange]}
   if {$xmax > 8192} then {set xmax 8192}
   if {$xmax < $xrange}    then {set xmax $xrange}
   adjustxlims
   do-display
}

proc click-left args {shift l}
proc click-right args {shift r}

proc adjustxlims {} {
  global xmin xmax xscale xrange
  set xscale [expr [expr $xmax - $xmin + $xrange - 1]/$xrange]
  set xmin [expr [expr $xmin / $xscale] * $xscale]
  set xmax [expr [expr $xscale * $xrange] + $xmin]
}


proc show_posn {n} {
  global cursor colours yrange

  :draw text :tag tg$n :delete T :colour [lindex $colours $n] :text "channel $cursor($n)" \
       :value [expr [expr $n * 80] - 30] $yrange

  foreach i {2 3}  {
     set time [expr [expr $cursor($i) - $cursor(1)] * 10]
     :draw text :tag tgm$i :delete T :colour [lindex $colours $i] :text "$time nsecs" \
         :value [expr [expr $i * 90] +  120] $yrange
  }
}

proc click-reset args {
   global xmin xmax xscale xrange
   set xmin 0
   set xmax 8192
   set xscale [expr [expr $xmax - $xmin]/$xrange]

   do-update
}

proc click-redisplay args {do-update}
 
proc click-hardcopy args {

   if {[midas-check-for-temp] == 0} then {return}

   set plotfile canvasdump.ps
   [:path TRACE] postscript -rotate yes -colormode mono -pageheight 200m -file $plotfile
   print-postscript $plotfile
}

proc do-display {} {
   global trigger_position cursor
   set-footer "displaying..."
   set cursor(4) $trigger_position
   for {set i 0} {$i<8} {incr i} {do_trace $i}
   for {set i 1} {$i<5} {incr i} {do_cursor $i}

   set-footer "traces updated at  [date]"
}

proc do_trace {n} {
  global trace tracetitles colours
  global xbase xrange yrange xscale xmin xmax
  global height

  set ybase [expr [expr [expr $n + 1] * $height] - 5]
  set low $ybase
  set high  [expr [expr $n * $height] + 5]

  set x 0
  set y 0

  set xc $xbase
  set yc $ybase

  :draw modify :tag t$n :delete T

  set i 0
  foreach s $trace($n) {
    incr x $s
    if {$y == $high} then {set y $low} else {set y $high}
    if {$x < $xmin} then {continue}
    if {$x > $xmax} then {set x $xmax}
    set xp [expr [expr [expr $x - $xmin]/$xscale] + $xbase]

    if {$i == 0} then {
      :draw line :tag t$n :colour red  :value $xbase $y $xc $y $xp $y
    } else {
      :draw line :tag t$n :colour red  :value $xc $yc $xc $y $xp $y
    }

    if {$x == $xmax} then {break}

    set xc $xp
    set yc $y
    incr i
  }

  if {$n == 0} then {
    :draw text :tag m0 :delete T :colour [first $colours] :text $xmin :value 5 $yrange
    :draw text :tag m1 :delete T :colour [first $colours] :text $xmax :value [expr $xrange - 20] $yrange
  }
}

proc test-traces {} {
  global trace

set trace(0) [list 4000 192 4000]
set trace(1) [list 4050 92 4050]
set trace(2) [list 3 60 8183]
set trace(3) [list 506 199 7487]
set trace(4) [list 503 199 7490]
set trace(5) [list 8192]
set trace(6) [list 0 5 8187]
set trace(7) [list 8192]
}

proc read-trace {n} {
   global resource trace RMchan trace_index
   set p [lindex $trace_index $n ]
   set v [EG "register :name '/$resource/RM$RMchan.LA.Trace$p' ?value"]
   if {$v == ""} then {set v 8192}
   set trace($n)   $v 
}
proc read-place {} {
   global resource  RMchan place trigger_position
   set v [EG "register :name '/$resource/RM$RMchan.LA.Trig.Place' ?value"]
   if {$v == ""} then {set v 8}
   set place [expr { 16 - [expr { $v & 15 }] } ]
   set trigger_position	[expr $place * 512 ]
}      

proc do-update {} {
  global started
   if {$started == 0} then {make-frame; set started 1} else {open-frame}

   set-footer "fetching..."
#   test-traces
   for {set i 0} {$i<8} {incr i} {read-trace $i}
   read-place
   do-display
}

proc do-refresh {} {
   global trigger_position cursor

   for {set i 0} {$i<8} {incr i} {read-trace $i}
   read-place
   set cursor(4) $trigger_position
   for {set i 0} {$i<8} {incr i} {do_trace $i}
   for {set i 1} {$i<5} {incr i} {do_cursor $i}

   set-footer "traces updated at  [date]"
}



