# 26-10-92 PHO
#	included status register while reg doed clear on read
# 14-10-92 PHO
#	get list of rocos automatically
# 13-10-92 PHO
#	added redisplay after reset
#	implemented help

set debug 0
set started 0

set frame ROCO
set frametitle ROCO
set icontitle ROCO

set help_topic ROCO

for {set i 1} {$i <= 3} {incr i} {lappend defaultchanlist [expr 0+$i]}

set cardtype ROCO
set VXIGlobal ""
set chantype ""

set chanlist {???}
set chanindex 0


#    The array "table" which is indexed by the "widget name" contains all the information needed
#    for the register/display item
#
#    Each element of "table" is a list
#
#       0.    name of global parameter used to hold current value of the register
#       1.    procedure to be used to convert the widget value into parameter value
#       2.    procedure to be used to convert the parameter value into a widget value
#       3.    name of the register
#       4.    procedure to be used to convert the register value into parameter value
#       5.    procedure to be used to convert the parameter value into a register value
#       6.    name of unit used for register access
#       7.    widget type to be used for "by parameter" displays
#       8-10  parameters for the procedure creating "by parameter" displays
#              eg  if (7) is :slider then 8 is min value, 9 is max value and 10 is unit
#       11    for startup only - initial value of the parameter (1)
#       12    for startup only - inserted for this widget into the table "parameternames"


set table(CHANNEL) {channel  widget->channel   channel->widget                 nil 
           nil            nil           nil nil nil nil nil 1 nil}
set table(IRF)     {irf      int-widget->param param->int-widget "ENF.InhibRq" 
          int-reg->param param->int-reg ""  nil nil nil nil 0 nil}
set table(ENABLE)  {enable   int-widget->param param->int-widget "Enable" 
          int-reg->param param->int-reg ""  nil nil nil nil 0 nil}
set table(IENF)    {ienf     int-widget->param param->int-widget "ENF.IntEnable" 
          int-reg->param param->int-reg ""  nil nil nil nil 0 nil}
set table(TEST)    {testmode int-widget->param param->int-widget "TestMode"
          int-reg->param param->int-reg ""  nil nil nil nil 0 nil}
set table(ITOR)    {itor     int-widget->param param->int-widget "Timeout.IntEnable"
          int-reg->param param->int-reg ""  nil nil nil nil 0 nil}
set table(EDTS)    {edts     int-widget->param param->int-widget "DTSTOP.Enable" 
         int-reg->param param->int-reg ""  nil nil nil nil 0 nil}
set table(TOR)     {tor      int-widget->param param->int-widget "Timeout.Status" 
         int-reg->param param->int-reg ""  nil nil nil nil 0 nil}
set table(ENF)     {enf      int-widget->param param->int-widget "ENF.Status" 
         int-reg->param param->int-reg ""  nil nil nil nil 0 nil}
set table(DTP)     {dtp      int-widget->param param->int-widget "DTRENtoDTPASS"
         int-reg->param param->int-reg ""  nil nil nil nil 0 nil}
set table(TIMEOUT) {timeout widget->param param->hex-str-widget "Timeouts"
         int-reg->param param->int-reg NIL nil nil nil nil 0 nil}
set table(INTREG)  {intbus widget->param param->hex-str-widget "InteruptBusLevel" 
         int-reg->param param->int-reg NIL nil nil nil nil 0 nil}
set table(FNE)     {fne int-widget->param param->int-widget "FIFOs.Empty"
         int-reg->param param->int-reg ""  nil nil nil nil 0 nil}
set table(DFF)     {dff int-widget->param param->int-widget "FIFO.Full"
         int-reg->param param->int-reg ""  nil nil nil nil 0 nil}
set table(EFF)     {enf int-widget->param param->int-widget "EvNumFIFO.Full" 
         int-reg->param param->int-reg ""  nil nil nil nil 0 nil}


loadsource common.vxi


proc get-roco-status {} {

    set m [read-vxi-reg "Status"]
    :modify STATUS :value "$m"
}
  
proc click-reset {} {

     write-vxi-reg "Reset" 1
     click-redisplay
}

proc click arg {
   global debug
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {return [click-quit]}
   switch $w  {
      REDISPLAY {click-redisplay}
      RESET {click-reset}
      HELP {click-help}
      TESTMODE {click-testmode $v}
      default {click-on $w $v}
   }
}


proc make-frame1 {} {
   global env chanlist

set col1 20c
set col2 25c

 :frame [self frame-name] :label "[self frame-title]" :show-footer T :resize F \
      :icon-image "@$env(ICONHOME)/vxi_module" \
      :icon-name "[self icon-title]"

    :panel PANEL1 :layout T :padx 5 :pady 5
     :layout H; :row-gap 10
      :choice-stack CHANNEL :label "ROCO Number:" :valuex 15c :strings $chanlist
        :next-row T

	:checkbox IRF  :label "Inhibit Request on ENF:" :valuex $col1
	:next-row T
	:checkbox ENABLE      :label "Enable:"          :valuex $col1
	:checkbox IENF :label "Interrupt on ENF:"       :valuex $col2
	:next-row T
	:checkbox TEST :label "Test:"                   :valuex $col1
	:checkbox ITOR :label "Interupt on Timeout:"    :valuex $col2
	:next-row T
	:checkbox EDTS :label "Enable DTSTOP:"          :valuex $col1
	:checkbox TOR  :label "Timeout on VXI Readout:" :valuex $col2
	:next-row T
	:checkbox ENF  :label "Event Number Fault:"     :valuex $col1
	:checkbox DTP  :label "DTREN to DTPASS:"        :valuex $col2
	:next-row T
	:choice FNE :label FIFOs: :strings [list "Not Empty" Empty] :deaf T :valuex $col1
	:next-row T
	:choice DFF :label "Data FIFO:" :strings [list "Not Full" Full] :deaf T :valuex $col1
	:next-row T
	:choice EFF :label "Event Number FIFO:" :strings [list "Not Full" Full] :deaf T :valuex $col1
        :hskip 10
	:text STATUS  :label "Status Register:"    :w 6 :read-only T 
	:next-row T
	:text TIMEOUT  :label "Timeouts Register:" :w 4 :valuex $col1
	:text INTREG   :label "Interrupt Bus Level:" :w 4 :valuex $col2
	:next-row T
	:button REDISPLAY :label Redisplay
	:button RESET     :label Reset
        :hskip 80
	:button HELP      :label Help...

   :show T
}
