# trigger.tcl---trigger card setup

#    March 26 1996   conversion for tk  VP
# Fri Jun 9 1995, VP
#   minimum value of Sumbus delay should be 16 nsecs
# Wed Jan 25 17:30:37 GMT 1995, DB
#   substituted $env(EXPBASE) for /eg
# 4-12-92 PHO
#	added 2 more digital inspection points 
# 9-11-92 PHO
#	added presets menu for logic input threshold 
# 23-10-92 PHO
#	put step on sliders [lindex table(xxx) 10]
# 15-10-92 PHO
#	made overflow buttons deaf
# 13-10-92 PHO
#	Implemented default option on load buttons on LCA frame
# 8-10-92 PHO
#	Implemented help
# 23-9-92 PHO
#	sumbus threshold range set back from 0 to 40 
#	scaler overflows and enable updated with scalers
# 22-9-92 PHO
#	Moved Slave/Master and Normal/Reset from from Electronics to LCA Window
# 17-9-92 PHO
#	sumbus threshold range set from 0 to 10
# 15-9-92 PHO
#	show scaler rates on redisplay if that was selected before
#	show input sumbus and logic groups selected on redisplay
# 14-9-92 PHO
#	compressed window to take up less screen space
#	redirected all register calls through write-vxi-reg and read-vxi-regs functions
#	caused updating on start and when new groups selected
#	re jigged scaler display
#	included feedback after writing register values
# Thu Jul 16 12:13:28 BST 1992, DB
#   added real-param->int-widget proc for conversion of multiplicity thresholds


set debug 0
set started 0

set frame TRIGGER
set frametitle "Master Trigger Control"
set icontitle "Trigger"

set cardtype Trig
set VXIGlobal ""

set showrates 0

set fwdirname $env(EXPBASE)/lca
set fwprogname ""
set fwdfltprogname $fwdirname/$fwprogname


set analogueinsptable \
  {"Disconnected" "RwGeSumbus" "ClGeSumbus" "BGOSumbus" "UserSumbus" 
   "LogicThresh" "TACout"}

set digitalinsptable \
  {"Disconnected" "RwGe1.Thresh" "ClGe1.Thresh" "BGO1.Thresh" "User1.Thresh"
   "RwGe1.Thresh+GD" "ClGe1.Thresh+GD" "BGO1.Thresh+GD" "User1.Thresh+GD"
   "FT" "FT+GD" "Val" "Val+GD" "Inhibit" "StartVal" "FPStop" "StartConv"
   "FRAM*" "AdrStrobe*" "Readout*" "NoneLeft" "MaxClock" "Pass*"
   "WrFIFO*" "TrigReadout"}

set insp_table(MUX-AI1) {"Insp.A1" AI}
set insp_table(MUX-AI2) {"Insp.A2" AI}
set insp_table(MUX-DI1) {"Insp.D1" DI}
set insp_table(MUX-DI2) {"Insp.D2" DI}
set insp_table(MUX-VI)  {"Insp.V"  VI}

set inspection_widgetnames {MUX-AI1 MUX-AI2 MUX-DI1 MUX-DI2 MUX-VI}
set inspection_regnames    {Insp.A1 Insp.A2 Insp.D1 Insp.D2 Insp.V}

#    The array "table" which is indexed by the "widget name" contains all the information needed
#    for the register/display item
#
#    Each element of "table" is a list
#
#       0.    name of global parameter used to hold current value of the register
#       1.    procedure to be used to convert the widget value into parameter value
#       2.    procedure to be used to convert the parameter value into a widget value
#       3.    name of the register
#       4.    procedure to be used to convert the register value into parameter value
#       5.    procedure to be used to convert the parameter value into a register value
#       6.    name of unit used for register access
#       7.    widget type to be used for "by parameter" displays
#       8-10  parameters for the procedure creating "by parameter" displays
#              eg  if (7) is :slider then 8 is min value, 9 is max value and 10 is unit
#       11    for startup only - initial value of the parameter (1)
#       12    for startup only - inserted for this widget into the table "parameternames"


set table(FTDELAY) {ftdelay slider-widget->param param->int-widget "FT.Delay"
       unit-reg->param param->unit-reg ns nilloc nil nil 8 1 nil}
set table(FTWIDTH) {ftwidth slider-widget->param param->int-widget "FT.Width"
       unit-reg->param param->unit-reg ns nilloc nil nil 4 1 nil}
set table(INHIBITDELAY) {inhibitdelay slider-widget->param param->int-widget "Inhibit.Delay"
       unit-reg->param param->unit-reg ns nilloc nil nil 4 1 nil}
set table(INHIBITWIDTH) {inhibitwidth slider-widget->param param->int-widget "Inhibit.Width"
       unit-reg->param param->unit-reg ns nilloc nil nil 47 1 nil}
set table(FTVALDELAY) {ftvaldelay slider-widget->param param->int-widget "FTVal.Delay"
       unit-reg->param param->unit-reg ns nilloc nil nil 38 1 nil}
set table(FTVALWIDTH) {ftvalwidth slider-widget->param param->int-widget "FTVal.Width"
       unit-reg->param param->unit-reg ns nilloc nil nil 38 1 nil}
set table(VALDELAY) {valdelay slider-widget->param param->int-widget "Val.Delay"
       unit-reg->param param->unit-reg ns nilloc nil nil 38 1 nil}
set table(VALWIDTH) {valwidth slider-widget->param param->int-widget "Val.Width"
       unit-reg->param param->unit-reg ns nilloc nil nil 38 1 nil}

foreach i {1 2 3 4} {
   set table(FTR$i-PRESCALE) "ftr[set i]prescale int-widget->param param->int-widget \"FTR$i.Prescale\"
       int-reg->param param->int-reg nil nilloc nil nil nil 1 nil"
}

foreach i {FTR1 FTR2 FTR3 FTR4 FT Val Test} {
   set table(SC-[uppercase $i]-ENABLE) "sc[set i]enable int-widget->param param->int-widget \"$i.Scaler.Enable\"
       int-reg->param param->int-reg nil nilloc nil nil nil 1 nil"
   set table(SC-[uppercase $i]-COUNT) "sc[set i]count str-widget->param param->str-widget \"$i.Scaler\"
       bigint-reg->param param->int-reg nil nilloc nil nil nil 1 nil"
   set table(SC-[uppercase $i]-OVERFLOW) "sc[set i]overflow int-widget->param param->int-widget \"$i.Scaler.Overflow\"
       int-reg->param param->int-reg nil nilloc nil nil nil 1 nil"
}

set table(LOGIC-THRESH) {logic-thresh slider-widget->param param->int-widget "FP.Input.Thresh"
       unit-reg->param param->unit-reg mV nilloc nil nil 3 1 nil}
set table(RAWGE-RD) {rawgerd int-widget->param param->int-widget "RwGe.Enable"
       int-reg->param param->int-reg nil nilloc nil nil nil 1 nil}
set table(RAWGE-ADDR) {rawgeaddr str-widget->param param->hex-str-widget "RwGe.Addr"
       int-reg->param param->int-reg nil nilloc nil nil nil 1 nil}
set table(CLEANGE-RD) {cleangerd int-widget->param param->int-widget "ClGe.Enable"
       int-reg->param param->int-reg nil nilloc nil nil nil 1 nil}
set table(CLEANGE-ADDR) {cleangeaddr str-widget->param param->hex-str-widget "ClGe.Addr"
       int-reg->param param->int-reg nil nilloc nil nil nil 1 nil}
set table(BGO-RD) {bgord int-widget->param param->int-widget "BGO.Enable"
       int-reg->param param->int-reg nil nilloc nil nil nil 1 nil}
set table(BGO-ADDR) {bgoaddr str-widget->param param->hex-str-widget "BGO.Addr"
       int-reg->param param->int-reg nil nilloc nil nil nil 1 nil}
set table(USER-RD) {userrd int-widget->param param->int-widget "User.Enable"
       int-reg->param param->int-reg nil nilloc nil nil nil 1 nil}
set table(USER-ADDR) {useraddr str-widget->param param->hex-str-widget "User.Addr"
       int-reg->param param->int-reg nil nilloc nil nil nil 1 nil}
set table(ETYPE-RD) {etyperd int-widget->param param->int-widget "FTVal.Enable"
       int-reg->param param->int-reg nil nilloc nil nil nil 1 nil}
set table(ETYPE-ADDR) {etypeaddr str-widget->param param->hex-str-widget "FTVal.Addr"
       int-reg->param param->int-reg nil nilloc nil nil nil 1 nil}
set table(TAC-RD) {tacrd int-widget->param param->int-widget "TAC.Enable"
       int-reg->param param->int-reg nil nilloc nil nil nil 1 nil}
set table(TAC-ADDR) {tacaddr str-widget->param param->hex-str-widget "TAC.Addr"
       int-reg->param param->int-reg nil nilloc nil nil nil 1 nil}

set table(SETUP-GE-CFD) {setupgecfd int-widget->param param->int-widget "GeCFD.Setup"
       int-reg->param param->int-reg nil nilloc nil nil nil 1 nil}
set table(SETUP-GE-TAC) {setupgetac int-widget->param param->int-widget "GeTAC.Setup"
       int-reg->param param->int-reg nil nilloc nil nil nil 1 nil}
set table(EXTERNALDELAY) {externaldelay int-widget->param param->int-widget "GeTAC.Setup"
       int-reg->param param->int-reg nil nilloc nil nil nil 1 nil}       
set table(SETUP-SELECT) {setupsumbus int-widget->param param->int-widget "CFD.SelectSB"
       int-reg->param param->int-reg nil nilloc nil nil nil 1 nil}

set table(LCA-VAL) {lcaval str-widget->param param->str-widget "Val.LCA"
       str-reg->param param->str-reg nil nilloc nil nil nil 1 nil}
set table(LCA-FT) {lcaft str-widget->param param->str-widget "FT.LCA"
       str-reg->param param->str-reg nil nilloc nil nil nil 1 nil}
set table(LCA-LOADED) {lcaloaded int-widget->param param->int-widget "LCA.Programmed"
       int-reg->param param->int-reg nil nilloc nil nil nil 1 nil}
set table(LCAMODE) {lcamode int-widget->param param->int-widget "LCA.Master" 
       int-reg->param param->int-reg nil nil nil nil 1 nil}
set table(LCARESET) {lcareset int-widget->param param->int-widget "LCA.Reset" 
       int-reg->param param->int-reg nil nil nil nil 1 nil}

#    table elements for "electronics" window

set table(LASTCARD) {lastcard int-widget->param param->int-widget "LastCard" 
   int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(TESTMODE) {testmode int-widget->param param->int-widget "TestMode" 
   int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(PASSED) {pass int-widget->param param->int-widget "Passed" 
   int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(MODELCODE) {modcode str-widget->param param->hex-widget "VXI.ModelCode"
   int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(SERIALNUM) {serialnum str-widget->param param->hex-widget "VXI.SerialNum"
   int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(MODLEVEL) {modlevel str-widget->param param->hex-widget "VXI.ModLevel"
   int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(MANID) {manid str-widget->param param->hex-widget "VXI.ManID"
   int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(OFFSET) {offset str-widget->param param->hex-widget "VXI.Offset"
   int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(VXIGOSTOP) {vxigostop int-widget->param param->int-widget "VXI.Go" 
   int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(INHIBIT) {inhibit int-widget->param param->int-widget "InhibitAction" 
   int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(READOUTSOURCE) {readoutsource int-widget->param param->int-widget "EndRO.Select" 
   int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(READOUTMODE) {readoutmode int-widget->param param->int-widget "ReadoutMode" 
   int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(FIFOEMPTY) {fifoempty int-widget->param param->int-widget "FIFO.NotEmpty" 
   int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(FIFOFULL) {fifofull int-widget->param param->int-widget "FIFO.NotFull" 
   int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(VXICLEAR) {vxiclear int-widget->param param->int-widget "VXI.Clear" 
   int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(VXITRIGSYNC) {vxitrigsync int-widget->param param->int-widget "VXI.Sync.Trig" 
   int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(SUMBUS-SELECT) {sumbusselect int-widget->param param->int-widget "Sumbus.Select" 
   int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(VAL-SELECT) {valselect int-widget->param param->int-widget "ValLine.Select"
   int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(SYNCMODE) {syncmode int-widget->param param->int-widget "ParallelMode" 
   int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(TESTDAC) {testdac slider-widget->param param->int-widget "Sumbus.Test" 
   unit-reg->param param->unit-reg mV nil nil nil 3 1 nil}


#    the remaining part of the table has a variable register name dependent on sumbus group and
#    logic group selected

proc update_table {} {
  global sumbusgrp logicgrp table

set table(RAWGETHRESH)   "rawgethresh int-widget->param real-param->int-widget \"RwGe$sumbusgrp.Thresh\"
       int-reg->param param->int-reg nil nil nil nil nil 1 nil"
set table(RAWGEDELAY)    "rawgedelay slider-widget->param param->int-widget \"RwGe$sumbusgrp.Delay\"
       unit-reg->param param->unit-reg ns nil nil nil 8 1 nil"
set table(RAWGEWIDTH)    "rawgewidth slider-widget->param param->int-widget \"RwGe$sumbusgrp.Width\"
       unit-reg->param param->unit-reg ns nil nil nil 13 1 nil"
set table(CLEANGETHRESH) "cleangethresh int-widget->param real-param->int-widget \"ClGe$sumbusgrp.Thresh\"
       int-reg->param param->int-reg nil nil nil nil nil 1 nil"
set table(CLEANGEDELAY)  "cleangedelay slider-widget->param param->int-widget \"ClGe$sumbusgrp.Delay\"
       unit-reg->param param->unit-reg ns nil nil nil 8 1 nil"
set table(CLEANGEWIDTH)  "cleangewidth slider-widget->param param->int-widget \"ClGe$sumbusgrp.Width\"
       unit-reg->param param->unit-reg ns nil nil nil 13 1 nil"
set table(BGOTHRESH)     "bgothresh int-widget->param real-param->int-widget \"BGO$sumbusgrp.Thresh\"
       int-reg->param param->int-reg nil nil nil nil nil 1 nil"
set table(BGODELAY)      "bgodelay slider-widget->param param->int-widget \"BGO$sumbusgrp.Delay\"
       unit-reg->param param->unit-reg ns nil nil nil 8 1 nil"
set table(BGOWIDTH)      "bgowidth slider-widget->param param->int-widget \"BGO$sumbusgrp.Width\"
       unit-reg->param param->unit-reg ns nil nil nil 13 1 nil"
set table(USERTHRESH)    "userthresh int-widget->param real-param->int-widget \"User$sumbusgrp.Thresh\"
       int-reg->param param->int-reg nil nil nil nil nil 1 nil"
set table(USERDELAY)     "userdelay slider-widget->param param->int-widget \"User$sumbusgrp.Delay\"
       unit-reg->param param->unit-reg ns nil nil nil 8 1 nil"
set table(USERWIDTH)     "userwidth slider-widget->param param->int-widget \"User$sumbusgrp.Width\"
       unit-reg->param param->unit-reg ns nil nil nil 13 1 nil"

set table(FT-IN-DELAY) "ftindelay slider-widget->param param->int-widget \"FP.FT$logicgrp.Delay\"
       unit-reg->param param->unit-reg ns nil nil nil 8 1 nil"
set table(FT-IN-WIDTH) "ftinwidth slider-widget->param param->int-widget \"FP.FT$logicgrp.Width\"
       unit-reg->param param->unit-reg ns nil nil nil 13 1 nil"
set table(VAL-IN-DELAY) "valindelay slider-widget->param param->int-widget \"FP.Val$logicgrp.Delay\"
       unit-reg->param param->unit-reg ns nil nil nil 38 1 nil"
set table(VAL-IN-WIDTH) "valinwidth slider-widget->param param->int-widget \"FP.Val$logicgrp.Width\"
       unit-reg->param param->unit-reg ns nil nil nil 38 1 nil"
}



#    register selection for parameter save function

set registersavepatterns {
   FT.Delay FT.Width Inhibit.Delay Inhibit.Width FTVal.Delay
   FTVal.Width Val.Delay Val.Width 
   RwGe[1-4].Thresh RwGe[1-4].Delay RwGe[1-4].Width
   ClGe[1-4].Thresh ClGe[1-4].Delay ClGe[1-4].Width 
   BGO[1-4].Thresh  BGO[1-4].Delay  BGO[1-4].Width 
   User[1-4].Thresh User[1-4].Delay User[1-4].Width 
   FTR[1-4].Prescale 
   FTR[1-4].Scaler.Enable  
   FT.Scaler.Enable Val.Scaler.Enable Test.Scaler.Enable 
   FP.FT[1-6].Delay  FP.FT[1-6].Width
   FP.Val[1-6].Delay FP.Val[1-6].Width 
   FP.Input.Thresh
   RwGe.Enable  ClGe.Enable 
   BGO.Enable  User.Enable FTVal.Enable TAC.Enable  
   GeCFD.Setup GeTAC.Setup Sumbus.Select
}


#    execute this procedure update_table immediately to create table items on startup

set sumbusgrp 1
set logicgrp 1

update_table

loadsource common.vxi

proc setregnames {} {
  global regnames table

  update_table

  set regnames {}
  set names [lsort [array names table]]
  foreach w $names {
     set line $table($w)
     if {[lindex $line 12] != "nil"} then {
        lappend regnames [lindex $line 3]
     }
  }
  foreach w $names {
     set line $table($w)
     if {[lindex $line 12] == "nil"} then {
        lappend regnames [lindex $line 3]
     }
  }
}

#    execute immediately to ensure registers start up complete

setregnames

#    procedures which select the sumbus group or logic group must update the register table

proc set-sumbus-group v {
   global sumbusgrp
   set sumbusgrp [expr $v+1]
   setregnames
   update-pairs [read-vxi-regs *$sumbusgrp*]
}

proc set-logic-group v {
   global logicgrp
   set logicgrp [expr $v+1]
   setregnames
   update-pairs [read-vxi-regs *$logicgrp*]
}


proc scan-bigint s {
# note return of ??? if s fails to scan
   if {[string first "0x" $s]==0} then \
      {set z [scan [string range $s 2 end] %x zzz]} \
   else {set z [scan $s %d zzz]}
   if {$z==1} then {return $zzz} else {return "???"}
}

proc bigint-reg->param {p v} {global $p; set $p [scan-bigint $v]}

proc click-lca-load {w m} {
   global fwprogname fwdirname
   switch $w {
	FT-MENU  {set reg FT.LCA}
	VAL-MENU {set reg Val.LCA}
	default {return}
   }

   if {$m=="Default" || $fwprogname==""} then {
	init-vxi-reg $reg
   } else {
	write-vxi-reg $reg $fwdirname/$fwprogname
   }
   click-lca-redisplay
}

proc click-lca-redisplay {} {
   global fwdirname fwprogname debug
   fw-repaint
   fw-paint-directory
   set m [read-vxi-regs *LCA*]
   update-pairs $m
}

proc click-redisplay {} {
   global env debug lcavisible showrates sumbusgrp logicgrp

   clear-footer
   set m [read-vxi-regs *]
   update-pairs $m
   :modify SC-REDISPLAY :value $showrates
   :modify INPUT-SELECT :value [expr $sumbusgrp-1]
   :modify IN-LOGIC-SELECT :value [expr $logicgrp-1]
   if {$showrates} then {scaler-rate}  else {redisplay-scalers}
   if {$lcavisible} then {click-lca-redisplay}
}   

proc click-redisplay-scalers val {
   global showrates
   set showrates $val
   if {$showrates} then {scaler-rate} else {redisplay-scalers}
}   

proc redisplay-scalers {} {
   write-vxi-reg Scalers.Xfer 1
   write-vxi-reg Scalers.Xfer 0 
   set m [read-vxi-regs *.Scaler*]
   update-pairs $m
   set-footer "Counts Updated [date]"
}   

proc click-clear-scalers {} {
   global env
   write-vxi-reg Scalers.Clear 1 
   write-vxi-reg Scalers.Clear 0
   :modify SC-REDISPLAY :value 0
   click-redisplay-scalers 0
}

proc scaler-rate {} {
   global env rate frame
   set period 3

   :modify $frame :busy T
   after [expr $period*1000] update-rate
   write-vxi-reg Scalers.Xfer 1
   write-vxi-reg Scalers.Xfer 0 
   set m [read-vxi-regs *.Scaler]
   foreach s $m {set rate([first $s]) [second $s]}

   set m [read-vxi-regs *.Scaler.*]
   update-pairs $m
   set-footer "Waiting for $period seconds..."
}

proc update-rate {} {
   global env rate frame period
   set period 3

   write-vxi-reg Scalers.Xfer 1
   write-vxi-reg Scalers.Xfer 0 
   set m [read-vxi-regs *.Scaler]

   set pp {}
   foreach s $m {
# note: sometimes scaler values are returned as empty strings
# d will be non-numeric if either scaler value is null
      catch {expr ((1*[second $s])-$rate([first $s]))/$period} d
      lappend pp [list [first $s] "$d"]
   }
   update-pairs $pp
   :modify $frame :busy F
   set-footer "Rates updated [date]"
}

proc click-lca {} {
   global lcavisible lcaframe
   if {$lcavisible==0} then {make-lca} else {open-frame $lcaframe}
   click-lca-redisplay
   set lcavisible 1
}   

proc click-logic v {
   global debug
   if {$debug} then {puts stdout "selecting logic input $v"}

   switch $v {
   TTL            {click-on LOGIC-THRESH 2000}
   {Fast NIM}     {click-on LOGIC-THRESH -600}
   ECL            {click-on LOGIC-THRESH -900}
   default        {}
   }
}

proc click-reset {} {
	clear-footer
	write-vxi-reg VXI.Reset 1
	init-vxi-reg *
	click-redisplay
}

proc click-reset-fifo {} {
	clear-footer
	write-vxi-reg FIFO.Reset 1
	write-vxi-reg FIFO.Reset 0
	click-redisplay
}

proc click-electronics-redisplay {} {
	update-pairs [read-regs *]
	update-fifo
}

proc update-fifo {} {
   global env fifoempty fifofull
      if {!$fifoempty}             then {:modify FIFO :value 0}
      if {!$fifofull}              then {:modify FIFO :value 2}
      if {$fifoempty && $fifofull} then {:modify FIFO :value 1}
}

proc click arg {
   global debug
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {return [click-quit $f]}
   switch -glob $w {
      REDISPLAY         {click-redisplay}
      ELECTRONICS       {click-electronics}
      RESET             {click-reset}
      RESET-FIFO        {click-reset-fifo}
      HELP              {click-help}
      LCA               {click-lca}
      FWDIR             {click-fw-dirname $v}
      FWPROG            {click-fw-progname $v}
      FWLIST            {click-fw-list $c $v}
      LCA-LOAD-VAL      -
      LCA-LOAD-FT       {}
      VAL-MENU          -
      FT-MENU           {click-lca-load $w $v}
      LCA-REDISPLAY     {click-lca-redisplay}
      LCA-HELP          {click-help}
      INPUT-SELECT      {set-sumbus-group $v}
      IN-LOGIC-SELECT   {set-logic-group $v}
      SC-REDISPLAY      {click-redisplay-scalers $v}
      SC-CLEAR          {click-clear-scalers}
      LOGIC-TYPE-SELECT {}
      LOGIC-TYPE-MENU   {click-logic $v}
      MUX-*  {click-inspection $w [lindex $arg 4]}
      default           {click-on $w $v}
   }
}


set lcaframe         "[self frame-name].lca"
set electronicsframe "[self frame-name].el"

proc make-frame1 {} {
   global env analogueinsptable digitalinsptable voltageinsptable

set col1 14c
set col2 6c
set col3 6c

 :frame [self frame-name] :label "[self frame-title]" :resize F :show-footer T \
      :icon-image "@$env(ICONHOME)/vxi_module" \
      :icon-name "[self icon-title]"

 :panel P1 :layout T :padx 5 :pady 5
   :layout H; :row-gap 5
     :choice-stack INPUT-SELECT :label "Input Sumbus Group:" :valuex 20c :strings [list 1 2 3 4]
     :hskip 100
     :choice-stack SETUP-SELECT :label "Sumbus:" :strings [list "Raw Ge" "Clean Ge" "BGO" "User"]
   :next-row T
     :message M1 :label "Delay ns"       :bold T :valuex 25c
     :message M2 :label "Width ns"       :bold T :valuex 35c
     :message M3 :label "Threshold mult" :bold T :valuex 40c
   :next-row T 
     :slider RAWGEDELAY  :label "Raw Ge:" :min 0 :max 2040 :valuex $col1
     :slider RAWGEWIDTH  :label ""        :min 0 :max 3200 :valuex $col2
     :slider RAWGETHRESH :label ""        :min 0 :max 40   :valuex $col3 :resolution 1
   :next-row T
     :slider CLEANGEDELAY  :label "Clean Ge:" :min 0 :max 2040 :valuex $col1
     :slider CLEANGEWIDTH  :label ""          :min 0 :max 3200 :valuex $col2
     :slider CLEANGETHRESH :label ""          :min 0 :max 40   :valuex $col3 :resolution 1
   :next-row T
     :slider BGODELAY  :label "BGO:" :min 0 :max 2040 :valuex $col1
     :slider BGOWIDTH  :label ""     :min 0 :max 3200 :valuex $col2
     :slider BGOTHRESH :label ""     :min 0 :max 40   :valuex $col3 :resolution 1
   :next-row T
     :slider USERDELAY  :label "User:" :min 0 :max 2040 :valuex $col1
     :slider USERWIDTH  :label ""      :min 0 :max 3200 :valuex $col2
     :slider USERTHRESH :label ""      :min 0 :max 40   :valuex $col3 :resolution 1

 :panel P2 :layout T :padx 5 :pady 5 
   :layout H; :row-gap 5
     :choice-stack IN-LOGIC-SELECT :label "Input Logic Group:" :valuex 20c :strings [list 1 2 3 4 5 6]
   :next-row T
     :message M4 :label "Delay ns" :bold T :valuex 25c
     :message M5 :label "Width ns" :bold T :valuex 35c
   :next-row T
     :slider FT-IN-DELAY :label "Fast Trigger:" :min 0 :max 2000 :valuex $col1
     :hskip 7
     :slider FT-IN-WIDTH :label ""              :min 0 :max 3200 :valuex $col2
   :next-row T
     :slider VAL-IN-DELAY :label "Validation:" :min 0 :max 10000 :valuex $col1
     :slider VAL-IN-WIDTH :label ""            :min 0 :max 10000 :valuex $col2
   :next-row T
     :slider LOGIC-THRESH :label "Threshold mV:" :min -5000 :max 5000 :valuex 17c
     :hskip 20
     :menubutton LOGIC-TYPE-MENU :label "Presets" :strings [list "TTL" "Fast NIM" "ECL"]
   :next-row T
     :message M6 :label "Output Pulse Timing:" :bold T :valuex 17c
     :message M7 :label "Delay ns"             :bold T :valuex 8c
     :message M8 :label "Width ns"             :bold T :valuex 35c
   :next-row T
     :slider FTDELAY :label "Fast Trigger:" :min 0 :max 2000 :valuex $col1
     :hskip 7
     :slider FTWIDTH :label ""              :min 0 :max 1000 :valuex $col2
   :next-row T
     :slider INHIBITDELAY :label "Inhibit:" :min 0 :max 1000  :valuex $col1
     :hskip 7
     :slider INHIBITWIDTH :label ""         :min 0 :max 12000 :valuex $col2
   :next-row T
     :slider FTVALDELAY :label "FT->Validation:" :min 0 :max 9700 :valuex $col1
     :slider FTVALWIDTH :label ""                :min 0 :max 9700 :valuex $col2
   :next-row T
     :slider VALDELAY :label "Validation:" :min 0 :max 9700 :valuex $col1
     :slider VALWIDTH :label ""            :min 0 :max 9700 :valuex $col2

   
 :panel P5 :layout R :padx 10 :after P1
   :layout H; :row-gap 0; :col-gap 0
   :vskip 20
     :message M12 :label "Readout Parameters:" :bold T :valuex 20c
   :next-row T
     :checkbox RAWGE-RD :label "Raw Ge Mult:" :valuex $col1
     :text RAWGE-ADDR   :label "" :w 5
   :next-row T
     :checkbox CLEANGE-RD :label "Clean Ge Mult:" :valuex $col1
     :text CLEANGE-ADDR   :label "" :w 5
   :next-row T
     :checkbox BGO-RD :label "BGO Mult:" :valuex $col1
     :text BGO-ADDR   :label "" :w 5
   :next-row T
     :checkbox USER-RD :label "User Mult:" :valuex $col1
     :text USER-ADDR   :label "" :w 5
   :next-row T
     :checkbox ETYPE-RD :label "Event Type:" :valuex $col1
     :text ETYPE-ADDR   :label "" :w 5
   :next-row T
     :checkbox TAC-RD :label "TAC:" :valuex $col1
     :text TAC-ADDR   :label "" :w 5
   :next-row T
   :layout V
     :vskip 20
     :message M13 :label "Timing Calibration:" :bold T :valuex 20c
     :checkbox SETUP-GE-CFD :label "Ge CFD:" :valuex $col1
     :checkbox SETUP-GE-TAC :label "Ge TAC:" :valuex $col1
     :checkbox EXTERNALDELAY :label "External delay:" :valuex $col1

   :next-row T
   :layout H
    :vskip 20
     :message M9          :label "Scalers:   Enable" :bold T 
     :choice SC-REDISPLAY :strings [list "Counts" "Rates"]
     :message M10         :label "Overflow" :bold T
   :next-row T
     :checkbox SC-FTR1-ENABLE   :label "FTR1:" :valuex $col1
     :text     SC-FTR1-COUNT    :label "" :w 12 :deaf T
     :checkbox SC-FTR1-OVERFLOW :label "" :deaf T
   :next-row T
     :checkbox SC-FTR2-ENABLE   :label "FTR2:" :valuex $col1
     :text     SC-FTR2-COUNT    :label "" :w 12 :deaf T
     :checkbox SC-FTR2-OVERFLOW :label "" :deaf T
   :next-row T
     :checkbox SC-FTR3-ENABLE   :label "FTR3:" :valuex $col1
     :text     SC-FTR3-COUNT    :label "" :w 12 :deaf T
     :checkbox SC-FTR3-OVERFLOW :label "" :deaf T
   :next-row T
     :checkbox SC-FTR4-ENABLE   :label "FTR4:" :valuex $col1
     :text     SC-FTR4-COUNT    :label "" :w 12 :deaf T
     :checkbox SC-FTR4-OVERFLOW :label "" :deaf T
   :next-row T
     :checkbox SC-FT-ENABLE   :label "Fast Trigger:" :valuex $col1
     :text     SC-FT-COUNT    :label "" :w 12 :deaf T
     :checkbox SC-FT-OVERFLOW :label "" :deaf T
   :next-row T
     :checkbox SC-VAL-ENABLE   :label "Validation:" :valuex $col1
     :text     SC-VAL-COUNT    :label "" :w 12 :deaf T
     :checkbox SC-VAL-OVERFLOW :label "" :deaf T
   :next-row T
     :checkbox SC-TEST-ENABLE   :label "Test:" :valuex $col1
     :text     SC-TEST-COUNT    :label "" :w 12 :deaf T
     :checkbox SC-TEST-OVERFLOW :label "" :deaf T
 
 :panel P4 :layout T :padx 5 :pady 5
      :layout H; :col-gap 10; :row-gap 10
      :message M15 :label "Inspection Lines:" :bold T
      :next-row T
      :menubutton MUX-AI1 :label "Analogue Inspection 1:" :w 20 :menu $analogueinsptable
      :message MUX-AI1-USE   :label ""
      :next-row T
      :menubutton MUX-AI2 :label "Analogue Inspection 2:" :w 20 :menu $analogueinsptable
      :message MUX-AI2-USE   :label ""
      :next-row T
      :menubutton MUX-DI1 :label "Digital Inspection 1:"  :w 20 :menu $digitalinsptable
      :message MUX-DI1-USE   :label ""
      :next-row T
      :menubutton MUX-DI2 :label "Digital Inspection 2:"  :w 20 :menu $digitalinsptable
      :message MUX-DI2-USE   :label ""
      :next-row T

     :message M14 :label "FTR Prescaling:" :bold T
     :next-row T
     :choice-stack FTR1-PRESCALE :label "FTR1:" :valuex $col2 \
         :strings [list "/1" "/2" "/4" "/8" "/16"]
     :choice-stack FTR2-PRESCALE :label "FTR2:" :valuex $col2 \
         :strings [list "/1" "/2" "/4" "/8" "/16"]
     :choice-stack FTR3-PRESCALE :label "FTR3:" :valuex $col2 \
         :strings [list "/1" "/2" "/4" "/8" "/16"]
     :choice-stack FTR4-PRESCALE :label "FTR4:" :valuex $col2 \
         :strings [list "/1" "/2" "/4" "/8" "/16"]

 :panel P6 :layout B :padx 10 :after FOOTER
   :layout H
    :button REDISPLAY   :label "Redisplay"
    :button SC-CLEAR    :label "Clear Scalers"
    :button ELECTRONICS :label "Electronics..."
    :button LCA         :label "LCAs..."
    :hskip 80
    :button HELP        :label "Help..."
 
 :show T
}

proc make-lca {} {
  global env lcaframe lcavisible fwdirname fwprogname 

 :frame $lcaframe :label "EG Trigger LCAs" :show-footer T :resize F  \
      :icon-image "@$env(ICONHOME)/vxi_module" \
      :icon-name "[self icon-title]LCA"

 wm protocol $lcaframe \
     WM_DELETE_WINDOW {global lcavisible; set lcavisible 0; destroy $lcaframe}

   :panel P10 :layout T :padx 5 :pady 5
     :layout H; :row-gap 5
      :text FWDIR :label "Directory:" :w 20  :valuex 11c :value "$fwdirname"
      :list FWLIST :rows 10 :cols 20
      :vskip -30
   :next-row T
      :text FWPROG :label "File:" :w 20 :valuex 11c :value "$fwprogname"
   :next-row T
      :text LCA-VAL :label "Validation:" :w 25 :read-only T :valuex 11c
      :menubutton VAL-MENU :label "Load" :strings [list "File" "Default"]
   :next-row T
      :text LCA-FT :label "Fast Trigger:" :w 25 :read-only T :valuex 11c
      :menubutton FT-MENU :label "Load"  :strings [list "File" "Default"]
   :next-row T; :col-gap 10
      :checkbox LCA-LOADED :label "Loaded:" :valuex 15c
      :choice LCAMODE  :label "" :strings [list "Slave" "Master"]
      :choice LCARESET :label "" :strings [list "Normal" "Reset"]
   :next-row T; :col-gap 0
      :button LCA-REDISPLAY :label "Redisplay"
      :hskip 80
      :button LCA-HELP :label "Help..." 

   :show T
 set lcavisible 1
}

proc make-electronics {} {
  global env electronicsframe electronicsvisible

 :frame $electronicsframe :label "Master Trigger Electronics" :show-footer T :resize F  \
      :icon-image "@$env(ICONHOME)/vxi_module" \
      :icon-name "[self icon-title]El"

 wm protocol $electronicsframe \
     WM_DELETE_WINDOW {global electronicsvisible; set electronicsvisible 0; destroy $electronicsframe}

    :panel P20 :layout T :padx 5 :pady 5
     :layout H; :row-gap 5
      :checkbox LASTCARD  :label "Last card:" :valuex 20c
      :text MODELCODE     :label "Model Code:" :w 6 :read-only T :valuex 30c
    :next-row T
      :checkbox TESTMODE  :label "Test Mode:"                           :valuex 20c
      :text SERIALNUM     :label "Serial Number:" :w 6 :read-only T     :valuex 30c
    :next-row T
      :text MODLEVEL      :label "Modification Level:" :w 6 :read-only T :valuex 20c
      :text MANID         :label "Manufacturer ID:" :w 6  :read-only T   :valuex 30c
    :next-row T
      :checkbox PASSED    :label "Passed:"                               :valuex 20c
      :text OFFSET        :label "Offset Register:" :w 6 :read-only T    :valuex 30c
    :next-row T
     :choice VXIGOSTOP :label VXI:  :strings [list Stop Go]             :valuex 20c
     :choice INHIBIT   :label Inhibit: :strings [list inactive active]  :valuex 28c
    :next-row T
     :choice READOUTSOURCE :label "Readout Complete:" :strings [list Readout Coding] :valuex 20c
     :choice READOUTMODE   :label "Readout Mode:" :strings [list Normal Test] :valuex 21c
    :next-row T
      :choice FIFO     :label FIFO: :strings [list Empty "Not Empty" Full] :valuex 20c
      :choice SYNCMODE :label Sync: :strings [list CDT Parallel]        :valuex 14c
    :next-row T
      :choice VXICLEAR    :label "VXI Clear line:"   :strings [list Normal Clear] :valuex 20c
      :hskip 20
      :choice VXITRIGSYNC :label "VXI Trigger Sync:" :strings [list Off On] :valuex 20c
    :next-row T
      :choice-stack SUMBUS-SELECT :label Sumbus: :strings [list "Raw Ge" "Clean Ge" BGO User] :valuex 20c
      :choice-stack VAL-SELECT    :label "Validation Line:"  \
         :strings [list None "Validation 1" "Validation 2" "Validation 3"]  :valuex 15c
    :row-gap 10
    :next-row T
      :slider TESTDAC :label "Sumbus Test (mV):" :min -5000 :max 0  :valuex 25c
    :next-row T; :col-gap 10
      :button REDISPLAY  :label Redisplay
      :button RESET      :label Reset
      :button RESET-FIFO :label "Reset FIFO"
      :hskip 80
      :button HELP       :label Help...

  :show T

 set electronicsvisible 1
}
