# vxisave.tcl---saving card setups

#     5 feb 1997
#    Save only task - initially created using saverestore as a base
#    restructure for TK  -  mar 31 1996  VP
#  26 Sept 95  VP
#   add Cluster
# 30-6-95  Vp
#   add CAMAC
# 9-9-94 PHO
#	Added NAM
# 5-8-93 PHO
#	Added clover
# Fri Aug  7 14:32:26 BST 1992, DB
#   on restores, report failed register writes in frame footer and give up
# Mon Aug 17 16:33:33 BST 1992, DB
#   changed patterns for save to ...[?,??]...

set debug 0
set started 0
set logging 0


set frame VXISAVE
set frametitle "Saving VXI Module Setups"
set icontitle "VXI Save"

if {[info exists env(VXIServer)] == 0} then {set vxiserver VXI}  \
   else {set vxiserver $env(VXIServer)}

set server $vxiserver
set RegisterServer $vxiserver

set file ""
set savedirectory ""
set directory ""
set selection {}
set lowchannel 0
set highchannel 0
set card 0
set cardtypes {}
set tasknames {}
set cardchoice {}
set cardtype  ""
set contents ""

set allchans 1
set appendmode 0

set msg1 "\nWarning - The whole Save will be abandoned\nLocate and correct error and start again"
set msg2 "Warning - Append mode will allow you to save module parameters in an existing save directory\n\
   This will overwrite an existing save file for a module without warning."
set msg3 "Are you sure you wish to delete a Save directory?\n\
   If you continue all parameters saved will be lost forever!!."

set vxicontrolstarted 0

set Configdir "$env(MIDASEXPBASE)"
catch {set Configdir "$env(EXPERIMENTBASE)"}

proc set-experiment {experimentdir} {
#    experiment has been joined (changed)
   global Configdir
   set Configdir $experimentdir
   start
}

proc record-current-save-file {w f} {
   global Configdir server

   set z [catch {open [file join $Configdir [set server]_current_[set w]] w} file]
   if {$z != 0} then {set-footer "$file"; return}
   puts $file "$f"
   close $file
}

#    proc VXImodule2crate taken from common.vxi.tcl
proc VXImodule2crate {module} {
  global vxicontrolstarted

#    obtain crate holding a specific module
#    this uses the task  "vxicontrol"

  if {$vxicontrolstarted == 0} then {
     set started [inform vxicontrol inquire started]
     if {$started == 0} then {inform vxicontrol start; send vxicontrol close-frame}
     set vxicontrolstarted 1
  }

  set crate [send vxicontrol VXImodule2crate $module]
  return $crate
}


proc repaint {} {
   global lowchannel highchannel card directory file allchans
   :modify LOW       :value $lowchannel
   :modify HIGH      :value $highchannel
   :modify CARD      :value $card
   :modify DIRECTORY :value $directory
   :modify FILENAME  :value $file
   :modify ALLCHANS :value $allchans
}

proc click-append {v} {
   global appendmode msg2
   set appendmode $v
   if {$v == 1} then {if {[midas-confirmation "$msg2"] == 0} then {set appendmode 0}}
   :modify APPEND :value $appendmode
}

proc click-directory name {
   global directory
   set directory $name
   click-redisplay
}

proc click-list {v i} {
   global selection  appendmode  directorylist directory file
   set selection $i  
    
   if {$appendmode == 1} then { 
      set target [lindex $directorylist $selection]
      set name [last $target]
      if {$name == "." || $name == ".."} then  {return}
      if {[file isdirectory [file join $directory $name]]} then {
         set file $name
         repaint
      }
   }
}

proc click-low v {
   global lowchannel highchannel allchans
   set lowchannel $v
   if {$lowchannel<0} then  {set lowchannel 0}
   if {$lowchannel>255} then {set lowchannel 255}
   if {$highchannel<$lowchannel} then {set highchannel $lowchannel}
   set allchans 0
   repaint
}

proc click-high v {
   global lowchannel highchannel allchans
   set highchannel $v
   if {$highchannel<0} then {set highchannel 0}
   if {$highchannel>255} then {set highchannel 255}
   if {$lowchannel>$highchannel} then {set lowchannel $highchannel}
   set allchans 0
   repaint
}

proc click-card v {
   global card cardtype cardtypes 
   set card $v
   set cardtype [lindex $cardtypes $v]
}

proc click-crate v {
   global vxicratelist vxicrate
   set vxicrate [lindex $vxicratelist $v]
}

set directory_list_option "lt"

proc click-redisplay {} {
   global directory selection
   read-directory $directory
   paint-directory $directory
   set selection {}
   repaint
}

proc click-up {} {
  global directory appendmode file
  set directory [directory-up $directory]
  if {$appendmode == 1} then {set file ""}
  click-redisplay
}

proc click-down {} {
  global directory selection appendmode file
  set directory [directory-down $directory $selection]
  if {$appendmode == 1} then {set file ""}
  click-redisplay
}

set abortflag 0
set saveinfo {}

proc saveVXIregisters {modules} {
   global debug logging EGreport moduledefinition savedirectory contents
   global server RegisterServer
   global msg1
   global abortflag
   global saveinfo

   set abortmsg1 "Do you really want to abort the whole save?\n\
                 Click on \"cancel\" if yes\n\
                 If you wish to continue with the save then click on \"continue\""
   set abortmsg2 "\n\nSAVE aborted at your request\n"

   set EGreport_save $EGreport
   if {$EGreport == 0} then {set EGreport -1}
   set-busy

   append contents "\nSaved modules:-"

   set abortflag 0
   :modify ABORT :inactive F

   foreach module $modules {
#    allow chance to abort save 
      update
      if {$abortflag == 1} then {
         if {[midas-confirmation "$abortmsg1"] == 0} then {append contents "$abortmsg2"; break}
         set abortflag 0
      }

      set taskname ""
      foreach item $moduledefinition {
         set moduletype [second $item]
         if {$moduletype != "" && [string match "$moduletype*" $module]} then {set taskname [third $item]; break}
      }
      if {$taskname == ""} then {continue}

      set server [VXImodule2crate $module]
      set RegisterServer $server

          set register /$server/$module.VXI.SerialNum
          set z [catch {eg "(register :name '$register' ?value)" } serialnum]
          if {$z != 0} then {set serialnum "Unknown"}

          if {$debug} then {puts stdout "saving module $module serial number $serialnum"}
          set-footer "saving module $module serial number $serialnum"
          if {$logging == 1} then {insert-log "saving module $module serial number $serialnum"}
          set registerstosave [inform $taskname Return_Save_Registers]
          if {$debug} then {puts stdout "saving $registerstosave"}
          if {$logging == 1} then {insert-log "$registerstosave"}
          set saveinfo {}
          foreach item $registerstosave {
             set register /$server/$module.$item
#             set-footer "saving parameter $register ..."
             if {$debug} then {puts stdout  "saving parameter $register ..."}
             set z [catch {eg "(register :name '$register' ?value)" } list]
             if {$z != 0} then {
#        error saving parameter
                 set errmsg "[eg "(sys :error $list ?message)"]"
                 set msg "Error saving register $register - $errmsg"
                 set reply [midas_dialog [self frame-name].midasDialog "Error Report for [self frame-title]" \
                     "$msg" "warning" 0 "continue"  "abort save"]
                 append contents "\n$msg\n"
                 switch $reply {
                    0    {append contents "Error ignored\n"; continue}
                    1    {append contents "\nSave aborted\n";clear-busy; return}
                 } 
              }
             set list [list $register $list]
             if {$list != ""} then {lappend saveinfo "$list"}
          }
#    save any information for this module into a file
#          if {$saveinfo != ""} then {
            append contents " $module (serial number $serialnum)"
            set savefile [file join $savedirectory $module]
            set z [catch {open $savefile w} file]
            if {$z != 0} then {midas-warning "Unable to create Save file $savefile\n$file$msg1"; return}
            puts $file "parameters saved [date] for module $module serial number $serialnum"

            foreach element $saveinfo {
               set module [first [split [first $element] .]]
               set register [string range [first $element] [expr [string length $module] + 1] end]
               set value [second $element]
               puts $file "$register  $value"
            }
            close $file
#         }
   }
   :modify ABORT :inactive T
   set EGreport $EGreport_save
   clear-busy
}


proc click-savebycrate {} {
   global directory file vxicrate appendmode
   global savedirectory contents
   global msg1
   global server RegisterServer

   set abortmsg3 "The VXI system is in the GO state.\
                 \nSome parameters may not save correctly if you continue.\
                 \nIt is recommended that you abort the SAVE, stop the VXI system and then try again.\
                 \n\nDo you wish to continue with the SAVE?"

#    check state of VXI system
   set VXIstate [inform vxicontrol read-vxistate]
   if {$VXIstate != 0} then {
      if {[midas-confirmation "$abortmsg3"] == 0} then {return}
   }

#    first create the directory which will hold the save
   set savedirectory [file join $directory $file]
   if {$appendmode == 0} then {
      set z [catch {:file mkdir $savedirectory} m]
      if {$z != 0} then {midas-warning "Unable to create Save directory $savedirectory\n$m$msg1"; return}
   } else {
      catch {:file mkdir $savedirectory} 
   }

   set contents "VXI parameter save created [date]\n"
   append contents "Save made by Crate\n"

   if {$vxicrate == "All"} then {
     set modules ""
     set crates [inform vxicontrol set VXICrateList]
     foreach crate $crates {
        append modules "[inform vxicontrol set VXIModuleList($crate)]"
        append modules " "
     }
     append contents "Saved all VXI crates in configuration:- $crates\n"
     append contents "VXI crate configuration contained modules:- $modules\n"
   } else {
     set modules [inform vxicontrol set VXIModuleList($vxicrate)]
     append contents "Saved VXI crate $vxicrate\n"
     append contends "VXI crate configuration contained modules:- $modules\n"
   }

#  Do the save
   set z [catch {saveVXIregisters $modules} m]

#  create and write save contents file
   set contentsfile [file join $savedirectory contents]
   if {$appendmode == 0} then {
      set zz [catch {open $contentsfile w} f]
   } else {
      set zz [catch {open $contentsfile a} f]
      if {$zz != 0} then {set zz [catch {open $contentsfile w} f]}
   }
   if {$zz != 0} then {midas-warning "Unable to create file $contentsfile\n$f"} else {
     puts $f $contents
     close $f
   }

#    change file access
#   exec /usr/bin/chmod 777 $savedirectory
#   exec /usr/bin/chmod -R 666 $savedirectory

   click-redisplay

#    finished - show the result
   if {$z != 0} then {midas-warning "Save failed: $m"; return}

   if {$vxicrate == "All"} then {
       set-footer "Saved all VXI crates in file directory $savedirectory"
       midas-information "Saved all VXI crates in file directory $savedirectory\n\n$contents"
   } else {
       set-footer "Saved VXI crate $vxicrate in file directory $savedirectory"
       midas-information "Saved VXI crate $vxicrate in file directory $savedirectory\n\n$contents"
   }

   if {$vxicrate == "All"} then {
      foreach crate $crates {
         set server $crate
         set RegisterServer $server
         record-current-save-file setup $savedirectory
         write-register NetVar_HasBeenRestored 1
      }
   }  else {
         record-current-save-file setup $savedirectory
         write-register NetVar_HasBeenRestored 1
   }

#    change the directory name for next save
   if {$appendmode == 0} then {click-time-stamp}
}

proc click-savebymodule {} {
   global directory file vxicrate cardtype cardtypes appendmode
   global allchans lowchannel highchannel
   global savedirectory contents
   global msg1

   set abortmsg3 "The VXI system is in the GO state.\
                 \nSome parameters may not save correctly if you continue.\
                 \nIt is recommended that you abort the SAVE, stop the VXI system and then try again.\
                 \n\nDo you wish to continue with the SAVE?"

#    check state of VXI system
   set VXIstate [inform vxicontrol read-vxistate]
   if {$VXIstate != 0} then {
      if {[midas-confirmation "$abortmsg3"] == 0} then {return}
   }

#    first create the directory which will hold the save
   set savedirectory [file join $directory $file]
   if {$appendmode == 0} then {
      set z [catch {:file mkdir $savedirectory} m]
      if {$z != 0} then {midas-warning "Unable to create Save directory $savedirectory\n$m$msg1"; return}
   } else {
      catch {:file mkdir $savedirectory}
   }

   set contents "VXI parameter save created [date]\n"
   append contents "Save made by Module\n"

   if {$cardtype == "All"} then {
     set required [lrange $cardtypes 1 end]
     set range_start 0
     set range_end 99
     append contents "Saved all VXI modules in configuration:- $required\n"
   } else {
     set required $cardtype
     if {$cardtype == "Trig" || $cardtype == "TriggerMK2"} then {
#     Trigger modules have no range
     } else {
        if {$allchans == 0} then {
           set range_start $lowchannel
           set range_end $highchannel
           append contents "Saved VXI module type $cardtype; channel range $lowchannel - $highchannel\n"
        } else {
           set range_start 0
           set range_end 99
           append contents "Saved VXI module type $cardtype; all channels\n"
        }
     }
   }

#    find all modules in the VXI configuration
   set available ""
   set crates [inform vxicontrol set VXICrateList]
   foreach crate $crates {
     append available "[inform vxicontrol set VXIModuleList($crate)]"
     append available " "
   }

#    test each available module to check if it meets the requirements
   set mm ""
   foreach module $required {
      foreach m $available {
         if {[string match $module* $m]} then {
            set index [string trimleft $m $module]
            if {$index == ""} then {
               append mm "$m "
            } else {
               if {$index >= $range_start && $index <= $range_end} then \
                 {append mm "$m " }
            }
         }
      }
   }

#  Do the save
   set z [catch {saveVXIregisters $mm} m]

#  create and write save contents file
   set contentsfile [file join $savedirectory contents]
   if {$appendmode == 0} then {
      set zz [catch {open $contentsfile w} f]
   } else {
      set zz [catch {open $contentsfile a} f]
      if {$zz != 0} then {set zz [catch {open $contentsfile w} f]}
   }
   if {$zz != 0} then {midas-warning "Unable to create file $contentsfile\n$f"} else {
     puts $f $contents
     close $f
   }

#    change file access
#   exec /usr/bin/chmod 777 $savedirectory
#   exec /usr/bin/chmod -R 666 $savedirectory

   click-redisplay

#    finished - show the result
   if {$z != 0} then {midas-warning "Save failed: $m"; return}

   set-footer "Saved VXI module(s) $cardtype in file directory $savedirectory"
   midas-information "Saved VXI module(s) $cardtype in file directory $savedirectory\n\n$contents"

#    change the directory name for next save
   if {$appendmode == 0} then {click-time-stamp}
}

proc click-time-stamp {} {
   global file
   set file "Save-[date +%Y%h%d-%H.%M.%S]"
   :modify FILENAME :value $file
}

proc click-filename {name} {
   global directory file

   if {[file exists [file join $directory $name]]} then \
      {midas-warning "File [file join $directory $name] already exists!!!"; return}

   set file $name
}

proc click-delete {} {
   global directorylist selection directory msg3
   if {[llength $selection]==0} then {
      set-footer "no file selected!!!"
      return
   }
   set target [lindex $directorylist $selection]
   set file [last $target]
   if {$file == "." || $file == ".."} then  {set-footer "You cannot delete $file"; return}

   if {[midas-confirmation "$msg3"] == 0} then {return}

   set-footer "deleting save $file..."

   set z [catch {:file delete -r -f [file join $directory $file]} m]
   if {$z != 0 || $m != ""} then {set-footer $m; return}
   click-redisplay
   set-footer "$file deleted"
}

proc click-rename {} {
   global directorylist selection directory
   upvar #0 file destnfile
   if {[llength $selection]==0} then {
      set-footer "no file selected!!!"
      return
   }
   set target [lindex $directorylist $selection]
   set savefile [last $target]
   if {$savefile == "." || $savefile == ".."} then  {set-footer "You cannot rename $savefile"; return}

   set-footer "renaming save $savefile..."

   set z [catch {:file rename [file join $directory $savefile] [file join $directory $destnfile]} m]
   if {$z != 0 || $m != ""} then {set-footer $m; return}
   click-redisplay
   set-footer "$savefile renamed as $destnfile"
}

proc click-current {} {
   global directorylist selection directory
   if {[llength $selection]==0} then {
      set-footer "no file selected!!!"
      return
   }
   set target [lindex $directorylist $selection]
   set savefile [last $target]
   if {$savefile == "." || $savefile == ".."} then  {set-footer "You cannot make $savefile current"; return}

   set-footer "copying save $savefile as Current"

   catch {:file delete -r -f [file join [$directory Current]}
   set z [catch {:file copy [file join $directory $savefile] [file join $directory Current]} m]
   if {$z != 0 || $m != ""} then {set-footer $m; return}
   click-redisplay
   set-footer "$savefile copied as Current"
}

proc click-pattern {} {
   global cardtypes tasknames cardtype
   if {$cardtype == "All"} then {set cards $cardtypes} else {set cards $cardtype}
   foreach card $cards {
      if {$card != "All"} then {
         set task [lindex $tasknames [lsearch $cardtypes $card]]
         set msg [inform $task inquire registersavepatterns]
         if {$msg == "" || $msg == "\n"} {set msg "No registers saved for module $card"}
         midas-report "Registers saved for $card\n$msg" 
      }
   }
}

proc click arg {
   global debug frame
   global allchans
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]
   if {$c==":quit"} then {return [click-quit $f]}
   switch $w {
      DIRECTORY       {click-directory $v}
      DIRECTORYLIST   {click-list $v $i}
      UP              {click-up}
      DOWN            {click-down}

      LOW             {click-low $v}
      HIGH            {click-high $v}
      ALLCHANS        {set allchans $v}
      CARD            {click-card $v}

      CRATE           {click-crate $v}
      PATTERN         {click-pattern}

      FILENAME        {click-filename $v}
      TIME-STAMP      {click-time-stamp}
      DELETE          {click-delete}
      RENAME          {click-rename}
      CURRENT         {click-current}

      SAVEMODULE      {click-savebymodule}
      SAVECRATE       {click-savebycrate}
      APPEND          {click-append $v}

      REDISPLAY       {click-redisplay}
      HELP            {click-help}

      default         {insert-debug "$frame: click called with $arg"}
   }
}

proc start args {
   global env started directory cardtypes cardtype tasknames cardchoice
   global moduledefinition vxicratelist vxicrate allchans

     if {$started == 0} then {make-frame} else {open-frame}
     set started 1

     set z [catch {set directory [file join $env(EXPERIMENTBASE) VXIsetups]} m]
     if {$z != 0} then {
        midas-warning "$m\nYou should probably Join an experiment!"
        close-frame
        return
     }

     if {![file exists $directory]} then {
        set z [catch {:file mkdir $directory} m]
        if {$z!=0} then {midas-warning "Cannot create directory $directory\n $m"}
#        :file chmod 777 $directoryopen
     }    
 

     set vxicratelist "All"
     append vxicratelist " " [inform vxicontrol inquire VXICrateList]

     set moduledefinition [inform vxicontrol inquire moduledefinition]
     set cardchoice {All}
     set cardtypes {All}
     set tasknames {All}
     foreach item $moduledefinition {
        if {[fourth $item] != "" && [second $item] != ""} then {
           lappend cardchoice [first $item]
           lappend cardtypes [second $item]
           lappend tasknames [third $item]
        }
     }
     set cardtype [first $cardtypes]
     :modify CARD :strings $cardchoice :value 0
     set vxicrate [first $vxicratelist]
     :modify CRATE :strings $vxicratelist :value 0

     click-time-stamp
     click-redisplay
}

proc click-quit args {close-frame}

proc finish args {destroy-frame}

proc halt-vxi {} {inform vxicontrol DAQ-halt}


proc make-frame {} {
  global env

    :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/vxi_module" \
      :icon-name "[self icon-title]" :resize F

      :panel P1 :layout T :padx 10 :pady 10
         :layout H; :row-gap 0; :col-gap 10

         :text DIRECTORY :label "Directory:" :w 60 :shifters T
         :button UP   :label Up
         :button DOWN :label Down

      :panel P2 :layout T :padx 10
         :list DIRECTORYLIST :exclusive T :rows 10 :cols 80

      :panel P3 :layout T :padx 10 :pady 10
        :layout H
        :col-gap 10; :row-gap 10
         :number LOW  :label "Low channel:"  :min 0 :max 99 :value 1 :w 4 
         :number HIGH :label "High channel:" :min 0 :max 99 :value 1 :w 4 
         :checkbox ALLCHANS  :label "All channels"
         :hskip 25
         :choice-stack CARD  :label "VXI module type:"
        :next-row T
         :choice-stack CRATE :label "VXI crate:"
        :hskip 15
         :button PATTERN     :label "Show Save Patterns"
        :hskip 15
         :checkbox APPEND     :label Append
         :button DELETE       :label "Delete Save Directory"
         :button RENAME       :label "Rename Save Directory"
        :next-row T
         :text FILENAME       :label "Save Directory:" :w 40
         :button TIME-STAMP   :label "Create directory name"
         :button CURRENT      :label "Copy Save as Current"
        :next-row T
         :button REDISPLAY     :label Redisplay
         :button SAVEMODULE    :label "Perform Save by Module"
         :button SAVECRATE     :label "Perform Save by VXI crate"
         :button ABORT :command "global abortflag; set abortflag 1" :inactive T
        :hskip 10
         :button VXIHALT      :label "HALT VXI"  :command halt-vxi
        :hskip 10
         :button HELP  :command click-help
      
      :show T
}
